import pytest

from dbt.tests.adapter.utils.data_types.base_data_type_macro import BaseDataTypeMacro


seeds__expected_csv = """string_col
"Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
""".lstrip()

models__actual_sql = """
select cast('Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum.'
as {{ type_string() }}) as string_col
"""


class BaseTypeString(BaseDataTypeMacro):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"expected.csv": seeds__expected_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {"actual.sql": self.interpolate_macro_namespace(models__actual_sql, "type_string")}


class TestTypeString(BaseTypeString):
    pass



from dbt.tests.adapter.utils.base_utils import BaseUtils
from dbt.tests.util import (
    check_relations_equal,
    get_relation_columns,
    run_dbt,
)


class BaseArrayUtils(BaseUtils):
    def assert_columns_equal(self, project, expected_cols, actual_cols):
        assert (
            expected_cols == actual_cols
        ), f"Type difference detected: {expected_cols} vs. {actual_cols}"

    def test_expected_actual(self, project):
        run_dbt(["build"])

        # check contents equal
        check_relations_equal(project.adapter, ["expected", "actual"])

        # check types equal
        expected_cols = get_relation_columns(project.adapter, "expected")
        actual_cols = get_relation_columns(project.adapter, "actual")
        print(f"Expected: {expected_cols}")
        print(f"Actual: {actual_cols}")
        self.assert_columns_equal(project, expected_cols, actual_cols)



import pytest

from dbt.tests.util import check_relations_equal, run_dbt


seeds__seed_csv = """id,first_name,last_name,email,gender,ip_address
1,Jack,Hunter,jhunter0@pbs.org,Male,59.80.20.168
2,Kathryn,Walker,kwalker1@ezinearticles.com,Female,194.121.179.35
3,Gerald,Ryan,gryan2@com.com,Male,11.3.212.243
4,Bonnie,Spencer,bspencer3@ameblo.jp,Female,216.32.196.175
5,Harold,Taylor,htaylor4@people.com.cn,Male,253.10.246.136
6,Jacqueline,Griffin,jgriffin5@t.co,Female,16.13.192.220
7,Wanda,Arnold,warnold6@google.nl,Female,232.116.150.64
8,Craig,Ortiz,cortiz7@sciencedaily.com,Male,199.126.106.13
9,Gary,Day,gday8@nih.gov,Male,35.81.68.186
10,Rose,Wright,rwright9@yahoo.co.jp,Female,236.82.178.100
11,Raymond,Kelley,rkelleya@fc2.com,Male,213.65.166.67
12,Gerald,Robinson,grobinsonb@disqus.com,Male,72.232.194.193
13,Mildred,Martinez,mmartinezc@samsung.com,Female,198.29.112.5
14,Dennis,Arnold,darnoldd@google.com,Male,86.96.3.250
15,Judy,Gray,jgraye@opensource.org,Female,79.218.162.245
16,Theresa,Garza,tgarzaf@epa.gov,Female,21.59.100.54
17,Gerald,Robertson,grobertsong@csmonitor.com,Male,131.134.82.96
18,Philip,Hernandez,phernandezh@adobe.com,Male,254.196.137.72
19,Julia,Gonzalez,jgonzalezi@cam.ac.uk,Female,84.240.227.174
20,Andrew,Davis,adavisj@patch.com,Male,9.255.67.25
21,Kimberly,Harper,kharperk@foxnews.com,Female,198.208.120.253
22,Mark,Martin,mmartinl@marketwatch.com,Male,233.138.182.153
23,Cynthia,Ruiz,cruizm@google.fr,Female,18.178.187.201
24,Samuel,Carroll,scarrolln@youtu.be,Male,128.113.96.122
25,Jennifer,Larson,jlarsono@vinaora.com,Female,98.234.85.95
26,Ashley,Perry,aperryp@rakuten.co.jp,Female,247.173.114.52
27,Howard,Rodriguez,hrodriguezq@shutterfly.com,Male,231.188.95.26
28,Amy,Brooks,abrooksr@theatlantic.com,Female,141.199.174.118
29,Louise,Warren,lwarrens@adobe.com,Female,96.105.158.28
30,Tina,Watson,twatsont@myspace.com,Female,251.142.118.177
31,Janice,Kelley,jkelleyu@creativecommons.org,Female,239.167.34.233
32,Terry,Mccoy,tmccoyv@bravesites.com,Male,117.201.183.203
33,Jeffrey,Morgan,jmorganw@surveymonkey.com,Male,78.101.78.149
34,Louis,Harvey,lharveyx@sina.com.cn,Male,51.50.0.167
35,Philip,Miller,pmillery@samsung.com,Male,103.255.222.110
36,Willie,Marshall,wmarshallz@ow.ly,Male,149.219.91.68
37,Patrick,Lopez,plopez10@redcross.org,Male,250.136.229.89
38,Adam,Jenkins,ajenkins11@harvard.edu,Male,7.36.112.81
39,Benjamin,Cruz,bcruz12@linkedin.com,Male,32.38.98.15
40,Ruby,Hawkins,rhawkins13@gmpg.org,Female,135.171.129.255
41,Carlos,Barnes,cbarnes14@a8.net,Male,240.197.85.140
42,Ruby,Griffin,rgriffin15@bravesites.com,Female,19.29.135.24
43,Sean,Mason,smason16@icq.com,Male,159.219.155.249
44,Anthony,Payne,apayne17@utexas.edu,Male,235.168.199.218
45,Steve,Cruz,scruz18@pcworld.com,Male,238.201.81.198
46,Anthony,Garcia,agarcia19@flavors.me,Male,25.85.10.18
47,Doris,Lopez,dlopez1a@sphinn.com,Female,245.218.51.238
48,Susan,Nichols,snichols1b@freewebs.com,Female,199.99.9.61
49,Wanda,Ferguson,wferguson1c@yahoo.co.jp,Female,236.241.135.21
50,Andrea,Pierce,apierce1d@google.co.uk,Female,132.40.10.209
"""

model_sql = """
{{
  config(
    materialized = "table",
    sort = 'first_name',
    dist = 'first_name'
  )
}}

select * from {{ this.schema }}.seed
"""


class BaseTableMaterialization:
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"seed.csv": seeds__seed_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {"materialized.sql": model_sql}

    def test_table_materialization_sort_dist_no_op(self, project):
        # basic table materialization test, sort and dist is not supported by postgres so the result table would still be same as input

        # check seed
        results = run_dbt(["seed"])
        assert len(results) == 1

        # check run
        results = run_dbt(["run"])
        assert len(results) == 1

        check_relations_equal(project.adapter, ["seed", "materialized"])


class TestTableMat(BaseTableMaterialization):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_last_day


class BaseLastDay(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_last_day.csv": fixture_last_day.seeds__data_last_day_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_last_day.yml": fixture_last_day.models__test_last_day_yml,
            "test_last_day.sql": self.interpolate_macro_namespace(
                fixture_last_day.models__test_last_day_sql, "last_day"
            ),
        }


class TestLastDay(BaseLastDay):
    pass



# macros
macro_test_alter_column_type = """
-- Macro to alter a column type
{% macro test_alter_column_type(model_name, column_name, new_column_type) %}
  {% set relation = ref(model_name) %}
  {{ alter_column_type(relation, column_name, new_column_type) }}
{% endmacro %}
"""

macro_test_is_type_sql = """
{% macro simple_type_check_column(column, check) %}
    {% if check == 'string' %}
        {{ return(column.is_string()) }}
    {% elif check == 'float' %}
        {{ return(column.is_float()) }}
    {% elif check == 'number' %}
        {{ return(column.is_number()) }}
    {% elif check == 'numeric' %}
        {{ return(column.is_numeric()) }}
    {% elif check == 'integer' %}
        {{ return(column.is_integer()) }}
    {% else %}
        {% do exceptions.raise_compiler_error('invalid type check value: ' ~ check) %}
    {% endif %}
{% endmacro %}

{% macro type_check_column(column, type_checks) %}
    {% set failures = [] %}
    {% for type_check in type_checks %}
        {% if type_check.startswith('not ') %}
            {% if simple_type_check_column(column, type_check[4:]) %}
                {% do log('simple_type_check_column got ', True) %}
                {% do failures.append(type_check) %}
            {% endif %}
        {% else %}
            {% if not simple_type_check_column(column, type_check) %}
                {% do failures.append(type_check) %}
            {% endif %}
        {% endif %}
    {% endfor %}
    {% if (failures | length) > 0 %}
        {% do log('column ' ~ column.name ~ ' had failures: ' ~ failures, info=True) %}
    {% endif %}
    {% do return((failures | length) == 0) %}
{% endmacro %}

{% test is_type(model, column_map) %}
    {% if not execute %}
        {{ return(None) }}
    {% endif %}
    {% if not column_map %}
        {% do exceptions.raise_compiler_error('test_is_type must have a column name') %}
    {% endif %}
    {% set columns = adapter.get_columns_in_relation(model) %}
    {% if (column_map | length) != (columns | length) %}
        {% set column_map_keys = (column_map | list | string) %}
        {% set column_names = (columns | map(attribute='name') | list | string) %}
        {% do exceptions.raise_compiler_error('did not get all the columns/all columns not specified:\n' ~ column_map_keys ~ '\nvs\n' ~ column_names) %}
    {% endif %}
    {% set bad_columns = [] %}
    {% for column in columns %}
        {% set column_key = (column.name | lower) %}
        {% if column_key in column_map %}
            {% set type_checks = column_map[column_key] %}
            {% if not type_checks %}
                {% do exceptions.raise_compiler_error('no type checks?') %}
            {% endif %}
            {% if not type_check_column(column, type_checks) %}
                {% do bad_columns.append(column.name) %}
            {% endif %}
        {% else %}
            {% do exceptions.raise_compiler_error('column key ' ~ column_key ~ ' not found in ' ~ (column_map | list | string)) %}
        {% endif %}
    {% endfor %}
    {% do log('bad columns: ' ~ bad_columns, info=True) %}
    {% for bad_column in bad_columns %}
      select '{{ bad_column }}' as bad_column
      {{ 'union all' if not loop.last }}
    {% endfor %}
      select * from (select 1 limit 0) as nothing
{% endtest %}
"""

# models/schema

model_sql = """
select
    1::smallint as smallint_col,
    2::integer as int_col,
    3::bigint as bigint_col,
    4.0::real as real_col,
    5.0::double precision as double_col,
    6.0::numeric as numeric_col,
    '7'::text as text_col,
    '8'::varchar(20) as varchar_col
"""

schema_yml = """
version: 2
models:
  - name: model
    data_tests:
      - is_type:
          column_map:
            smallint_col: ['integer', 'number']
            int_col: ['integer', 'number']
            bigint_col: ['integer', 'number']
            real_col: ['float', 'number']
            double_col: ['float', 'number']
            numeric_col: ['numeric', 'number']
            text_col: ['string', 'not number']
            varchar_col: ['string', 'not number']
"""



from collections import namedtuple
from typing import Set

# TODO: repoint to dbt-artifacts when it's available
from dbt.artifacts.schemas.results import TestStatus
import pytest

from dbt.tests.adapter.store_test_failures_tests import _files
from dbt.tests.util import run_dbt, check_relation_types


TestResult = namedtuple("TestResult", ["name", "status", "type"])


class StoreTestFailuresAsBase:
    seed_table: str = "chipmunks_stage"
    model_table: str = "chipmunks"
    audit_schema_suffix: str = "dbt_test__audit"

    audit_schema: str

    @pytest.fixture(scope="class", autouse=True)
    def setup_class(self, project):
        # the seed doesn't get touched, load it once
        run_dbt(["seed"])
        yield

    @pytest.fixture(scope="function", autouse=True)
    def setup_method(self, project, setup_class):
        # make sure the model is always right
        run_dbt(["run"])

        # the name of the audit schema doesn't change in a class, but this doesn't run at the class level
        self.audit_schema = f"{project.test_schema}_{self.audit_schema_suffix}"
        yield

    @pytest.fixture(scope="function", autouse=True)
    def teardown_method(self, project):
        yield

        # clear out the audit schema after each test case
        with project.adapter.connection_named("__test"):
            audit_schema = project.adapter.Relation.create(
                database=project.database, schema=self.audit_schema
            )
            project.adapter.drop_schema(audit_schema)

    @pytest.fixture(scope="class")
    def seeds(self):
        return {f"{self.seed_table}.csv": _files.SEED__CHIPMUNKS}

    @pytest.fixture(scope="class")
    def models(self):
        return {f"{self.model_table}.sql": _files.MODEL__CHIPMUNKS}

    def run_and_assert(
        self, project, expected_results: Set[TestResult], expect_pass: bool = False
    ) -> None:
        """
        Run `dbt test` and assert the results are the expected results

        Args:
            project: the `project` fixture; needed since we invoke `run_dbt`
            expected_results: the expected results of the tests as instances of TestResult
            expect_pass: passed directly into `run_dbt`; this is only needed if all expected results are tests that pass

        Returns:
            the row count as an integer
        """
        # run the tests
        results = run_dbt(["test"], expect_pass=expect_pass)

        # show that the statuses are what we expect
        actual = {(result.node.name, result.status) for result in results}
        expected = {(result.name, result.status) for result in expected_results}
        assert actual == expected

        # show that the results are persisted in the correct database objects
        check_relation_types(
            project.adapter, {result.name: result.type for result in expected_results}
        )


class StoreTestFailuresAsInteractions(StoreTestFailuresAsBase):
    """
    These scenarios test interactions between `store_failures` and `store_failures_as` at the model level.
    Granularity (e.g. setting one at the project level and another at the model level) is not considered.

    Test Scenarios:

    - If `store_failures_as = "view"` and `store_failures = True`, then store the failures in a view.
    - If `store_failures_as = "view"` and `store_failures = False`, then store the failures in a view.
    - If `store_failures_as = "view"` and `store_failures` is not set, then store the failures in a view.
    - If `store_failures_as = "table"` and `store_failures = True`, then store the failures in a table.
    - If `store_failures_as = "table"` and `store_failures = False`, then store the failures in a table.
    - If `store_failures_as = "table"` and `store_failures` is not set, then store the failures in a table.
    - If `store_failures_as = "ephemeral"` and `store_failures = True`, then do not store the failures.
    - If `store_failures_as = "ephemeral"` and `store_failures = False`, then do not store the failures.
    - If `store_failures_as = "ephemeral"` and `store_failures` is not set, then do not store the failures.
    - If `store_failures_as` is not set and `store_failures = True`, then store the failures in a table.
    - If `store_failures_as` is not set and `store_failures = False`, then do not store the failures.
    - If `store_failures_as` is not set and `store_failures` is not set, then do not store the failures.
    """

    @pytest.fixture(scope="class")
    def tests(self):
        return {
            "view_unset_pass.sql": _files.TEST__VIEW_UNSET_PASS,  # control
            "view_true.sql": _files.TEST__VIEW_TRUE,
            "view_false.sql": _files.TEST__VIEW_FALSE,
            "view_unset.sql": _files.TEST__VIEW_UNSET,
            "table_true.sql": _files.TEST__TABLE_TRUE,
            "table_false.sql": _files.TEST__TABLE_FALSE,
            "table_unset.sql": _files.TEST__TABLE_UNSET,
            "ephemeral_true.sql": _files.TEST__EPHEMERAL_TRUE,
            "ephemeral_false.sql": _files.TEST__EPHEMERAL_FALSE,
            "ephemeral_unset.sql": _files.TEST__EPHEMERAL_UNSET,
            "unset_true.sql": _files.TEST__UNSET_TRUE,
            "unset_false.sql": _files.TEST__UNSET_FALSE,
            "unset_unset.sql": _files.TEST__UNSET_UNSET,
        }

    def test_tests_run_successfully_and_are_stored_as_expected(self, project):
        expected_results = {
            TestResult("view_unset_pass", TestStatus.Pass, "view"),  # control
            TestResult("view_true", TestStatus.Fail, "view"),
            TestResult("view_false", TestStatus.Fail, "view"),
            TestResult("view_unset", TestStatus.Fail, "view"),
            TestResult("table_true", TestStatus.Fail, "table"),
            TestResult("table_false", TestStatus.Fail, "table"),
            TestResult("table_unset", TestStatus.Fail, "table"),
            TestResult("ephemeral_true", TestStatus.Fail, None),
            TestResult("ephemeral_false", TestStatus.Fail, None),
            TestResult("ephemeral_unset", TestStatus.Fail, None),
            TestResult("unset_true", TestStatus.Fail, "table"),
            TestResult("unset_false", TestStatus.Fail, None),
            TestResult("unset_unset", TestStatus.Fail, None),
        }
        self.run_and_assert(project, expected_results)


class StoreTestFailuresAsProjectLevelOff(StoreTestFailuresAsBase):
    """
    These scenarios test that `store_failures_as` at the model level takes precedence over `store_failures`
    at the project level.

    Test Scenarios:

    - If `store_failures = False` in the project and `store_failures_as = "view"` in the model,
    then store the failures in a view.
    - If `store_failures = False` in the project and `store_failures_as = "table"` in the model,
    then store the failures in a table.
    - If `store_failures = False` in the project and `store_failures_as = "ephemeral"` in the model,
    then do not store the failures.
    - If `store_failures = False` in the project and `store_failures_as` is not set,
    then do not store the failures.
    """

    @pytest.fixture(scope="class")
    def tests(self):
        return {
            "results_view.sql": _files.TEST__VIEW_UNSET,
            "results_table.sql": _files.TEST__TABLE_UNSET,
            "results_ephemeral.sql": _files.TEST__EPHEMERAL_UNSET,
            "results_unset.sql": _files.TEST__UNSET_UNSET,
        }

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"data_tests": {"store_failures": False}}

    def test_tests_run_successfully_and_are_stored_as_expected(self, project):
        expected_results = {
            TestResult("results_view", TestStatus.Fail, "view"),
            TestResult("results_table", TestStatus.Fail, "table"),
            TestResult("results_ephemeral", TestStatus.Fail, None),
            TestResult("results_unset", TestStatus.Fail, None),
        }
        self.run_and_assert(project, expected_results)


class StoreTestFailuresAsProjectLevelView(StoreTestFailuresAsBase):
    """
    These scenarios test that `store_failures_as` at the project level takes precedence over `store_failures`
    at the model level.

    Test Scenarios:

    - If `store_failures_as = "view"` in the project and `store_failures = False` in the model,
    then store the failures in a view.
    - If `store_failures_as = "view"` in the project and `store_failures = True` in the model,
    then store the failures in a view.
    - If `store_failures_as = "view"` in the project and `store_failures` is not set,
    then store the failures in a view.
    """

    @pytest.fixture(scope="class")
    def tests(self):
        return {
            "results_true.sql": _files.TEST__VIEW_TRUE,
            "results_false.sql": _files.TEST__VIEW_FALSE,
            "results_unset.sql": _files.TEST__VIEW_UNSET,
        }

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"data_tests": {"store_failures_as": "view"}}

    def test_tests_run_successfully_and_are_stored_as_expected(self, project):
        expected_results = {
            TestResult("results_true", TestStatus.Fail, "view"),
            TestResult("results_false", TestStatus.Fail, "view"),
            TestResult("results_unset", TestStatus.Fail, "view"),
        }
        self.run_and_assert(project, expected_results)


class StoreTestFailuresAsProjectLevelEphemeral(StoreTestFailuresAsBase):
    """
    This scenario tests that `store_failures_as` at the project level takes precedence over `store_failures`
    at the model level. In particular, setting `store_failures_as = "ephemeral"` at the project level
    turns off `store_failures` regardless of the setting of `store_failures` anywhere. Turning `store_failures`
    back on at the model level requires `store_failures_as` to be set at the model level.

    Test Scenarios:

    - If `store_failures_as = "ephemeral"` in the project and `store_failures = True` in the project,
    then do not store the failures.
    - If `store_failures_as = "ephemeral"` in the project and `store_failures = True` in the project and the model,
    then do not store the failures.
    - If `store_failures_as = "ephemeral"` in the project and `store_failures_as = "view"` in the model,
    then store the failures in a view.
    """

    @pytest.fixture(scope="class")
    def tests(self):
        return {
            "results_unset.sql": _files.TEST__UNSET_UNSET,
            "results_true.sql": _files.TEST__UNSET_TRUE,
            "results_view.sql": _files.TEST__VIEW_UNSET,
        }

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"data_tests": {"store_failures_as": "ephemeral", "store_failures": True}}

    def test_tests_run_successfully_and_are_stored_as_expected(self, project):
        expected_results = {
            TestResult("results_unset", TestStatus.Fail, None),
            TestResult("results_true", TestStatus.Fail, None),
            TestResult("results_view", TestStatus.Fail, "view"),
        }
        self.run_and_assert(project, expected_results)


class StoreTestFailuresAsGeneric(StoreTestFailuresAsBase):
    """
    This tests that `store_failures_as` works with generic tests.
    Test Scenarios:

    - If `store_failures_as = "view"` is used with the `not_null` test in the model, then store the failures in a view.
    """

    @pytest.fixture(scope="class")
    def models(self):
        return {
            f"{self.model_table}.sql": _files.MODEL__CHIPMUNKS,
            "schema.yml": _files.SCHEMA_YML,
        }

    def test_tests_run_successfully_and_are_stored_as_expected(self, project):
        expected_results = {
            # `store_failures` unset, `store_failures_as = "view"`
            TestResult("not_null_chipmunks_name", TestStatus.Pass, "view"),
            # `store_failures = False`, `store_failures_as = "table"`
            TestResult(
                "accepted_values_chipmunks_name__alvin__simon__theodore", TestStatus.Fail, "table"
            ),
            # `store_failures = True`, `store_failures_as = "view"`
            TestResult("not_null_chipmunks_shirt", TestStatus.Fail, "view"),
        }
        self.run_and_assert(project, expected_results)


class StoreTestFailuresAsExceptions(StoreTestFailuresAsBase):
    """
    This tests that `store_failures_as` raises exceptions in appropriate scenarios.
    Test Scenarios:

    - If `store_failures_as = "error"`, a helpful exception is raised.
    """

    @pytest.fixture(scope="class")
    def tests(self):
        return {
            "store_failures_as_error.sql": _files.TEST__ERROR_UNSET,
        }

    def test_tests_run_unsuccessfully_and_raise_appropriate_exception(self, project):
        results = run_dbt(["test"], expect_pass=False)
        assert len(results) == 1
        result = results[0]
        assert "Compilation Error" in result.message
        assert "'error' is not a valid value" in result.message
        assert "Accepted values are: ['ephemeral', 'table', 'view']" in result.message



# escape_single_quotes

models__test_escape_single_quotes_quote_sql = """
select
  '{{ escape_single_quotes("they're") }}' as actual,
  'they''re' as expected,
  {{ length(string_literal(escape_single_quotes("they're"))) }} as actual_length,
  7 as expected_length

union all

select
  '{{ escape_single_quotes("they are") }}' as actual,
  'they are' as expected,
  {{ length(string_literal(escape_single_quotes("they are"))) }} as actual_length,
  8 as expected_length
"""


# The expected literal is 'they\'re'. The second backslash is to escape it from Python.
models__test_escape_single_quotes_backslash_sql = """
select
  '{{ escape_single_quotes("they're") }}' as actual,
  'they\\'re' as expected,
  {{ length(string_literal(escape_single_quotes("they're"))) }} as actual_length,
  7 as expected_length

union all

select
  '{{ escape_single_quotes("they are") }}' as actual,
  'they are' as expected,
  {{ length(string_literal(escape_single_quotes("they are"))) }} as actual_length,
  8 as expected_length
"""


models__test_escape_single_quotes_yml = """
version: 2
models:
  - name: test_escape_single_quotes
    data_tests:
      - assert_equal:
          actual: actual
          expected: expected
      - assert_equal:
          actual: actual_length
          expected: expected_length
"""



import pytest

from dbt.tests.util import write_file, run_dbt

my_model_sql = """
select
    tested_column from {{ ref('my_upstream_model')}}
"""

my_upstream_model_sql = """
select
  {sql_value} as tested_column
"""

test_my_model_yml = """
unit_tests:
  - name: test_my_model
    model: my_model
    given:
      - input: ref('my_upstream_model')
        rows:
          - {{ tested_column: {yaml_value} }}
    expect:
      rows:
        - {{ tested_column: {yaml_value} }}
"""


class BaseUnitTestingTypes:
    @pytest.fixture
    def data_types(self):
        # sql_value, yaml_value
        return [
            ["1", "1"],
            ["'1'", "1"],
            ["true", "true"],
            ["DATE '2020-01-02'", "2020-01-02"],
            ["TIMESTAMP '2013-11-03 00:00:00-0'", "2013-11-03 00:00:00-0"],
            ["TIMESTAMPTZ '2013-11-03 00:00:00-0'", "2013-11-03 00:00:00-0"],
            ["'1'::numeric", "1"],
            [
                """'{"bar": "baz", "balance": 7.77, "active": false}'::json""",
                """'{"bar": "baz", "balance": 7.77, "active": false}'""",
            ],
            # TODO: support complex types
            # ["ARRAY['a','b','c']", """'{"a", "b", "c"}'"""],
            # ["ARRAY[1,2,3]", """'{1, 2, 3}'"""],
        ]

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "my_model.sql": my_model_sql,
            "my_upstream_model.sql": my_upstream_model_sql,
            "schema.yml": test_my_model_yml,
        }

    def test_unit_test_data_type(self, project, data_types):
        for sql_value, yaml_value in data_types:
            # Write parametrized type value to sql files
            write_file(
                my_upstream_model_sql.format(sql_value=sql_value),
                "models",
                "my_upstream_model.sql",
            )

            # Write parametrized type value to unit test yaml definition
            write_file(
                test_my_model_yml.format(yaml_value=yaml_value),
                "models",
                "schema.yml",
            )

            results = run_dbt(["run", "--select", "my_upstream_model"])
            assert len(results) == 1

            try:
                run_dbt(["test", "--select", "my_model"])
            except Exception:
                raise AssertionError(f"unit test failed when testing model with {sql_value}")


class TestPostgresUnitTestingTypes(BaseUnitTestingTypes):
    pass



from collections import Counter
from copy import deepcopy
import os
import shutil

from dbt_common.exceptions import DbtRuntimeError
import pytest

from dbt.tests.adapter.dbt_clone import fixtures
from dbt.tests.util import run_dbt, run_dbt_and_capture


class BaseClone:
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "table_model.sql": fixtures.table_model_sql,
            "view_model.sql": fixtures.view_model_sql,
            "ephemeral_model.sql": fixtures.ephemeral_model_sql,
            "schema.yml": fixtures.schema_yml,
            "exposures.yml": fixtures.exposures_yml,
        }

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "macros.sql": fixtures.macros_sql,
            "infinite_macros.sql": fixtures.infinite_macros_sql,
            "get_schema_name.sql": fixtures.get_schema_name_sql,
        }

    @pytest.fixture(scope="class")
    def seeds(self):
        return {
            "seed.csv": fixtures.seed_csv,
        }

    @pytest.fixture(scope="class")
    def snapshots(self):
        return {
            "snapshot.sql": fixtures.snapshot_sql,
        }

    @pytest.fixture(scope="class")
    def other_schema(self, unique_schema):
        return unique_schema + "_other"

    @property
    def project_config_update(self):
        return {
            "seeds": {
                "test": {
                    "quote_columns": False,
                }
            }
        }

    @pytest.fixture(scope="class")
    def profiles_config_update(self, dbt_profile_target, unique_schema, other_schema):
        outputs = {"default": dbt_profile_target, "otherschema": deepcopy(dbt_profile_target)}
        outputs["default"]["schema"] = unique_schema
        outputs["otherschema"]["schema"] = other_schema
        return {"test": {"outputs": outputs, "target": "default"}}

    def copy_state(self, project_root):
        state_path = os.path.join(project_root, "state")
        if not os.path.exists(state_path):
            os.makedirs(state_path)
        shutil.copyfile(
            f"{project_root}/target/manifest.json", f"{project_root}/state/manifest.json"
        )

    def run_and_save_state(self, project_root, with_snapshot=False):
        results = run_dbt(["seed"])
        assert len(results) == 1
        assert not any(r.node.deferred for r in results)
        results = run_dbt(["run"])
        assert len(results) == 2
        assert not any(r.node.deferred for r in results)
        results = run_dbt(["test"])
        assert len(results) == 2

        if with_snapshot:
            results = run_dbt(["snapshot"])
            assert len(results) == 1
            assert not any(r.node.deferred for r in results)

        # copy files
        self.copy_state(project_root)


# -- Below we define base classes for tests you import the one based on if your adapter uses dbt clone or not --
class BaseClonePossible(BaseClone):
    def test_can_clone_true(self, project, unique_schema, other_schema):
        project.create_test_schema(other_schema)
        self.run_and_save_state(project.project_root, with_snapshot=True)

        clone_args = [
            "clone",
            "--state",
            "state",
            "--target",
            "otherschema",
        ]

        results = run_dbt(clone_args)
        assert len(results) == 4

        schema_relations = project.adapter.list_relations(
            database=project.database, schema=other_schema
        )
        types = [r.type for r in schema_relations]
        count_types = Counter(types)
        assert count_types == Counter({"table": 3, "view": 1})

        # objects already exist, so this is a no-op
        results = run_dbt(clone_args)
        assert len(results) == 4
        assert all("no-op" in r.message.lower() for r in results)

        # recreate all objects
        results = run_dbt([*clone_args, "--full-refresh"])
        assert len(results) == 4

        # select only models this time
        results = run_dbt([*clone_args, "--resource-type", "model"])
        assert len(results) == 2
        assert all("no-op" in r.message.lower() for r in results)

    def test_clone_no_state(self, project, unique_schema, other_schema):
        project.create_test_schema(other_schema)
        self.run_and_save_state(project.project_root, with_snapshot=True)

        clone_args = [
            "clone",
            "--target",
            "otherschema",
        ]

        with pytest.raises(
            DbtRuntimeError,
            match="--state or --defer-state are required for deferral, but neither was provided",
        ):
            run_dbt(clone_args)


class BaseCloneNotPossible(BaseClone):
    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "macros.sql": fixtures.macros_sql,
            "my_can_clone_tables.sql": fixtures.custom_can_clone_tables_false_macros_sql,
            "infinite_macros.sql": fixtures.infinite_macros_sql,
            "get_schema_name.sql": fixtures.get_schema_name_sql,
        }

    def test_can_clone_false(self, project, unique_schema, other_schema):
        project.create_test_schema(other_schema)
        self.run_and_save_state(project.project_root, with_snapshot=True)

        clone_args = [
            "clone",
            "--state",
            "state",
            "--target",
            "otherschema",
        ]

        results = run_dbt(clone_args)
        assert len(results) == 4

        schema_relations = project.adapter.list_relations(
            database=project.database, schema=other_schema
        )
        assert all(r.type == "view" for r in schema_relations)

        # objects already exist, so this is a no-op
        results = run_dbt(clone_args)
        assert len(results) == 4
        assert all("no-op" in r.message.lower() for r in results)

        # recreate all objects
        results = run_dbt([*clone_args, "--full-refresh"])
        assert len(results) == 4

        # select only models this time
        results = run_dbt([*clone_args, "--resource-type", "model"])
        assert len(results) == 2
        assert all("no-op" in r.message.lower() for r in results)


class TestPostgresCloneNotPossible(BaseCloneNotPossible):
    @pytest.fixture(autouse=True)
    def clean_up(self, project):
        yield
        with project.adapter.connection_named("__test"):
            relation = project.adapter.Relation.create(
                database=project.database, schema=f"{project.test_schema}_seeds"
            )
            project.adapter.drop_schema(relation)

            relation = project.adapter.Relation.create(
                database=project.database, schema=project.test_schema
            )
            project.adapter.drop_schema(relation)

    pass


class BaseCloneSameTargetAndState(BaseClone):
    def test_clone_same_target_and_state(self, project, unique_schema, other_schema):
        project.create_test_schema(other_schema)
        self.run_and_save_state(project.project_root)

        clone_args = [
            "clone",
            "--state",
            "target",
        ]

        results, output = run_dbt_and_capture(clone_args, expect_pass=False)
        assert "Warning: The state and target directories are the same: 'target'" in output


class TestCloneSameTargetAndState(BaseCloneSameTargetAndState):
    pass



from typing import Dict, List

from dbt.tests.fixtures.project import TestProjInfo
from dbt.tests.util import relation_from_name


def get_records(
    project: TestProjInfo, table: str, select: str = None, where: str = None  # type: ignore
) -> List[tuple]:
    """
    Gets records from a single table in a dbt project

    Args:
        project: the dbt project that contains the table to get records from
        table: the name of the table without a schema
        select: the selection clause; defaults to all columns (*)
        where: the where clause to apply, if any; defaults to all records

    Returns:
        A list of records with each record as a tuple
    """
    table_name = relation_from_name(project.adapter, table)
    select_clause = select or "*"
    where_clause = where or "1 = 1"
    sql = f"""
        select {select_clause}
        from {table_name}
        where {where_clause}
    """
    return [tuple(record) for record in project.run_sql(sql, fetch="all")]


def update_records(
    project: TestProjInfo, table: str, updates: Dict[str, str], where: str = None  # type: ignore
):
    """
    Applies updates to a table in a dbt project

    Args:
        project: the dbt project that contains the table
        table: the name of the table without a schema
        updates: the updates to be applied in the form {'field_name': 'expression to be applied'}
        where: the where clause to apply, if any; defaults to all records
    """
    table_name = relation_from_name(project.adapter, table)
    set_clause = ", ".join(
        [" = ".join([field, expression]) for field, expression in updates.items()]
    )
    where_clause = where or "1 = 1"
    sql = f"""
        update {table_name}
        set {set_clause}
        where {where_clause}
    """
    project.run_sql(sql)


def insert_records(
    project: TestProjInfo, to_table: str, from_table: str, select: str, where: str = None  # type: ignore
):
    """
    Inserts records from one table into another table in a dbt project

    Args:
        project: the dbt project that contains the table
        to_table: the name of the table, without a schema, in which the records will be inserted
        from_table: the name of the table, without a schema, which contains the records to be inserted
        select: the selection clause to apply on `from_table`; defaults to all columns (*)
        where: the where clause to apply on `from_table`, if any; defaults to all records
    """
    to_table_name = relation_from_name(project.adapter, to_table)
    from_table_name = relation_from_name(project.adapter, from_table)
    select_clause = select or "*"
    where_clause = where or "1 = 1"
    sql = f"""
        insert into {to_table_name}
        select {select_clause}
        from {from_table_name}
        where {where_clause}
    """
    project.run_sql(sql)


def delete_records(project: TestProjInfo, table: str, where: str = None):  # type: ignore
    """
    Deletes records from a table in a dbt project

    Args:
        project: the dbt project that contains the table and where to
        table: the name of the table without a schema
        where: the where clause to apply, if any; defaults to all records
    """
    table_name = relation_from_name(project.adapter, table)
    where_clause = where or "1 = 1"
    sql = f"""
        delete from {table_name}
        where {where_clause}
    """
    project.run_sql(sql)


def clone_table(
    project: TestProjInfo, to_table: str, from_table: str, select: str, where: str = None  # type: ignore
):
    """
    Creates a new table based on another table in a dbt project

    Args:
        project: the dbt project that contains the table
        to_table: the name of the table, without a schema, to be created
        from_table: the name of the table, without a schema, to be cloned
        select: the selection clause to apply on `from_table`; defaults to all columns (*)
        where: the where clause to apply on `from_table`, if any; defaults to all records
    """
    to_table_name = relation_from_name(project.adapter, to_table)
    from_table_name = relation_from_name(project.adapter, from_table)
    select_clause = select or "*"
    where_clause = where or "1 = 1"
    sql = f"drop table if exists {to_table_name}"
    project.run_sql(sql)
    sql = f"""
        create table {to_table_name} as
        select {select_clause}
        from {from_table_name}
        where {where_clause}
    """
    project.run_sql(sql)


def add_column(project: TestProjInfo, table: str, column: str, definition: str):
    """
    Applies updates to a table in a dbt project

    Args:
        project: the dbt project that contains the table
        table: the name of the table without a schema
        column: the name of the new column
        definition: the definition of the new column, e.g. 'varchar(20) default null'
    """
    # BigQuery doesn't like 'varchar' in the definition
    if project.adapter.type() == "bigquery" and "varchar" in definition.lower():
        definition = "string"
    table_name = relation_from_name(project.adapter, table)
    sql = f"""
        alter table {table_name}
        add column {column} {definition}
    """
    project.run_sql(sql)



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_get_powers_of_two


class BaseGetPowersOfTwo(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_get_powers_of_two.yml": fixture_get_powers_of_two.models__test_get_powers_of_two_yml,
            "test_get_powers_of_two.sql": self.interpolate_macro_namespace(
                fixture_get_powers_of_two.models__test_get_powers_of_two_sql, "get_powers_of_two"
            ),
        }


class TestGetPowersOfTwo(BaseGetPowersOfTwo):
    pass



import re

import pytest

from dbt.tests.util import check_relation_has_expected_schema, run_dbt


_MODEL_CURRENT_TIMESTAMP = """
select {{ current_timestamp() }} as current_timestamp,
       {{ current_timestamp_in_utc_backcompat() }} as current_timestamp_in_utc_backcompat,
       {{ current_timestamp_backcompat() }} as current_timestamp_backcompat
"""

_MODEL_EXPECTED_SQL = """
select now() as current_timestamp,
       (current_timestamp at time zone 'utc')::TIMESTAMP as current_timestamp_in_utc_backcompat,
       current_timestamp::TIMESTAMP as current_timestamp_backcompat
"""


class BaseCurrentTimestamps:
    @pytest.fixture(scope="class")
    def models(self):
        return {"get_current_timestamp.sql": _MODEL_CURRENT_TIMESTAMP}

    # any adapters that don't want to check can set expected schema to None
    @pytest.fixture(scope="class")
    def expected_sql(self):
        return _MODEL_EXPECTED_SQL

    @pytest.fixture(scope="class")
    def expected_schema(self):
        return {
            "current_timestamp": "timestamp with time zone",
            "current_timestamp_in_utc_backcompat": "timestamp without time zone",
            "current_timestamp_backcompat": "timestamp without time zone",
        }

    def test_current_timestamps(self, project, models, expected_schema, expected_sql):
        results = run_dbt(["run"])
        assert len(results) == 1
        check_relation_has_expected_schema(
            project.adapter,
            relation_name="get_current_timestamp",
            expected_schema=expected_schema,
        )

        if expected_sql:
            generated_sql = results.results[0].node.compiled_code
            generated_sql_check = re.sub(r"\s+", "", generated_sql).lower()
            expected_sql_check = re.sub(r"\s+", "", expected_sql).lower()
            assert (
                expected_sql_check == generated_sql_check
            ), f"generated sql did not match expected: {generated_sql}"


class TestCurrentTimestamps(BaseCurrentTimestamps):
    pass



import pytest

from dbt.tests.adapter.column_types import fixtures
from dbt.tests.util import run_dbt


class BaseColumnTypes:
    @pytest.fixture(scope="class")
    def macros(self):
        return {"test_is_type.sql": fixtures.macro_test_is_type_sql}

    def run_and_test(self):
        results = run_dbt(["run"])
        assert len(results) == 1
        results = run_dbt(["test"])
        assert len(results) == 1


class BasePostgresColumnTypes(BaseColumnTypes):
    @pytest.fixture(scope="class")
    def models(self):
        return {"model.sql": fixtures.model_sql, "schema.yml": fixtures.schema_yml}

    def test_run_and_test(self, project):
        self.run_and_test()


class TestPostgresColumnTypes(BasePostgresColumnTypes):
    pass



# length

seeds__data_length_csv = """expression,output
abcdef,6
fishtown,8
december,8
www.google.com/path,19
"""


models__test_length_sql = """
with data as (

    select * from {{ ref('data_length') }}

)

select

    {{ length('expression') }} as actual,
    output as expected

from data
"""


models__test_length_yml = """
version: 2
models:
  - name: test_length
    data_tests:
      - assert_equal:
          actual: actual
          expected: expected
"""



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_listagg


class BaseListagg(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {
            "data_listagg.csv": fixture_listagg.seeds__data_listagg_csv,
            "data_listagg_output.csv": fixture_listagg.seeds__data_listagg_output_csv,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_listagg.yml": fixture_listagg.models__test_listagg_yml,
            "test_listagg.sql": self.interpolate_macro_namespace(
                fixture_listagg.models__test_listagg_sql, "listagg"
            ),
        }


class TestListagg(BaseListagg):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_string_literal


class BaseStringLiteral(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_string_literal.yml": fixture_string_literal.models__test_string_literal_yml,
            "test_string_literal.sql": self.interpolate_macro_namespace(
                fixture_string_literal.models__test_string_literal_sql, "string_literal"
            ),
        }


class TestStringLiteral(BaseStringLiteral):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_hash


class BaseHash(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_hash.csv": fixture_hash.seeds__data_hash_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_hash.yml": fixture_hash.models__test_hash_yml,
            "test_hash.sql": self.interpolate_macro_namespace(
                fixture_hash.models__test_hash_sql, "hash"
            ),
        }


class TestHash(BaseHash):
    pass



import os
import re

# TODO: does this belong in dbt-tests-adapters?
from dbt.cli.exceptions import DbtUsageException
import pytest
import yaml

from dbt.tests.util import run_dbt, run_dbt_and_capture


MODELS__MODEL_SQL = """
seled 1 as id
"""


class BaseDebug:
    @pytest.fixture(scope="class")
    def models(self):
        return {"model.sql": MODELS__MODEL_SQL}

    @pytest.fixture(autouse=True)
    def capsys(self, capsys):
        self.capsys = capsys

    def assertGotValue(self, linepat, result):
        found = False
        output = self.capsys.readouterr().out
        for line in output.split("\n"):
            if linepat.match(line):
                found = True
                assert result in line
        if not found:
            with pytest.raises(Exception) as exc:
                msg = f"linepat {linepat} not found in stdout: {output}"
                assert msg in str(exc.value)

    def check_project(self, splitout, msg="ERROR invalid"):
        for line in splitout:
            if line.strip().startswith("dbt_project.yml file"):
                assert msg in line
            elif line.strip().startswith("profiles.yml file"):
                assert "ERROR invalid" not in line


class BaseDebugProfileVariable(BaseDebug):
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"config-version": 2, "profile": '{{ "te" ~ "st" }}'}


class BaseDebugPostgres(BaseDebug):
    def test_ok(self, project):
        run_dbt(["debug"])
        assert "ERROR" not in self.capsys.readouterr().out

    def test_connection_flag(self, project):
        """Testing that the --connection flag works as expected, including that output is not lost"""
        _, out = run_dbt_and_capture(["debug", "--connection"])
        assert "Skipping steps before connection verification" in out

        _, out = run_dbt_and_capture(
            ["debug", "--connection", "--target", "NONE"], expect_pass=False
        )
        assert "1 check failed" in out
        assert "The profile 'test' does not have a target named 'NONE'." in out

        _, out = run_dbt_and_capture(
            ["debug", "--connection", "--profiles-dir", "NONE"], expect_pass=False
        )
        assert "Using profiles dir at NONE"
        assert "1 check failed" in out
        assert "dbt looked for a profiles.yml file in NONE" in out

    def test_nopass(self, project):
        run_dbt(["debug", "--target", "nopass"], expect_pass=False)
        self.assertGotValue(re.compile(r"\s+profiles\.yml file"), "ERROR invalid")

    def test_wronguser(self, project):
        run_dbt(["debug", "--target", "wronguser"], expect_pass=False)
        self.assertGotValue(re.compile(r"\s+Connection test"), "ERROR")

    def test_empty_target(self, project):
        run_dbt(["debug", "--target", "none_target"], expect_pass=False)
        self.assertGotValue(re.compile(r"\s+output 'none_target'"), "misconfigured")


class TestDebugPostgres(BaseDebugPostgres):
    pass


class BaseDebugInvalidProjectPostgres(BaseDebug):
    def test_empty_project(self, project):
        with open("dbt_project.yml", "w") as f:  # noqa: F841
            pass

        run_dbt(["debug", "--profile", "test"], expect_pass=False)
        splitout = self.capsys.readouterr().out.split("\n")
        self.check_project(splitout)

    def test_badproject(self, project):
        update_project = {"invalid-key": "not a valid key so this is bad project"}

        with open("dbt_project.yml", "w") as f:
            yaml.safe_dump(update_project, f)

        run_dbt(["debug", "--profile", "test"], expect_pass=False)
        splitout = self.capsys.readouterr().out.split("\n")
        self.check_project(splitout)

    def test_not_found_project(self, project):
        with pytest.raises(DbtUsageException):
            run_dbt(["debug", "--project-dir", "nopass"])

    def test_invalid_project_outside_current_dir(self, project):
        # create a dbt_project.yml
        project_config = {"invalid-key": "not a valid key in this project"}
        os.makedirs("custom", exist_ok=True)
        with open("custom/dbt_project.yml", "w") as f:
            yaml.safe_dump(project_config, f, default_flow_style=True)
        run_dbt(["debug", "--project-dir", "custom"], expect_pass=False)
        splitout = self.capsys.readouterr().out.split("\n")
        self.check_project(splitout)

    def test_profile_not_found(self, project):
        _, out = run_dbt_and_capture(
            ["debug", "--connection", "--profile", "NONE"], expect_pass=False
        )
        assert "Profile loading failed for the following reason" in out
        assert "Could not find profile named 'NONE'" in out


class TestDebugInvalidProjectPostgres(BaseDebugInvalidProjectPostgres):
    pass



SEED__CHIPMUNKS = """
name,shirt
alvin,red
simon,blue
theodore,green
dave,
""".strip()


MODEL__CHIPMUNKS = """
{{ config(materialized='table') }}
select *
from {{ ref('chipmunks_stage') }}
"""


TEST__VIEW_TRUE = """
{{ config(store_failures_as="view", store_failures=True) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__VIEW_FALSE = """
{{ config(store_failures_as="view", store_failures=False) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__VIEW_UNSET = """
{{ config(store_failures_as="view") }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__TABLE_TRUE = """
{{ config(store_failures_as="table", store_failures=True) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__TABLE_FALSE = """
{{ config(store_failures_as="table", store_failures=False) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__TABLE_UNSET = """
{{ config(store_failures_as="table") }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__EPHEMERAL_TRUE = """
{{ config(store_failures_as="ephemeral", store_failures=True) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__EPHEMERAL_FALSE = """
{{ config(store_failures_as="ephemeral", store_failures=False) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__EPHEMERAL_UNSET = """
{{ config(store_failures_as="ephemeral") }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__UNSET_TRUE = """
{{ config(store_failures=True) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__UNSET_FALSE = """
{{ config(store_failures=False) }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__UNSET_UNSET = """
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


TEST__VIEW_UNSET_PASS = """
{{ config(store_failures_as="view") }}
select *
from {{ ref('chipmunks') }}
where shirt = 'purple'
"""


TEST__ERROR_UNSET = """
{{ config(store_failures_as="error") }}
select *
from {{ ref('chipmunks') }}
where shirt = 'green'
"""


SCHEMA_YML = """
version: 2

models:
  - name: chipmunks
    columns:
      - name: name
        data_tests:
          - not_null:
              store_failures_as: view
          - accepted_values:
              store_failures: false
              store_failures_as: table
              values:
                - alvin
                - simon
                - theodore
      - name: shirt
        data_tests:
          - not_null:
              store_failures: true
              store_failures_as: view
"""



from typing import Dict, Iterable, List

import pytest

from dbt.tests.adapter.simple_snapshot import common, seeds, snapshots
from dbt.tests.util import run_dbt


MODEL_FACT_SQL = """
{{ config(materialized="table") }}
select * from {{ ref('seed') }}
where id between 1 and 20
"""


class BaseSimpleSnapshotBase:
    @pytest.fixture(scope="class")
    def seeds(self):
        """
        This seed file contains all records needed for tests, including records which will be inserted after the
        initial snapshot. This table will only need to be loaded once at the class level. It will never be altered, hence requires no further
        setup or teardown.
        """
        return {"seed.csv": seeds.SEED_CSV}

    @pytest.fixture(scope="class")
    def models(self):
        """
        This will be the working base table. It will be altered by each test, hence will require setup and
        teardown at the test case level. See `self._setup_method(self, project)`.
        """
        return {"fact.sql": MODEL_FACT_SQL}

    @pytest.fixture(scope="class", autouse=True)
    def _setup_class(self, project):
        """
        Load `seed` once for the whole class
        """
        run_dbt(["seed"])

    @pytest.fixture(scope="function", autouse=True)
    def _setup_method(self, project):
        """
        Initialize `fact` and `snapshot` for every test case.
        Only load the first 20 `seed` records into `fact`; withhold 10 records as "new" (e.g. to test inserts).

        Make the project a class variable to simplify function calls and make the code more readable.
        For some reason, this doesn't work in the class-scoped fixture, but does in the function-scoped fixture.
        """
        self.project = project
        self.create_fact_from_seed("id between 1 and 20")
        run_dbt(["snapshot"])
        yield
        self.delete_snapshot_records()
        self.delete_fact_records()

    def update_fact_records(self, updates: Dict[str, str], where: str = None):  # type: ignore
        common.update_records(self.project, "fact", updates, where)

    def insert_fact_records(self, where: str = None):  # type: ignore
        common.insert_records(self.project, "fact", "seed", "*", where)

    def delete_fact_records(self, where: str = None):  # type: ignore
        common.delete_records(self.project, "fact", where)

    def add_fact_column(self, column: str = None, definition: str = None):  # type: ignore
        common.add_column(self.project, "fact", column, definition)

    def create_fact_from_seed(self, where: str = None):  # type: ignore
        common.clone_table(self.project, "fact", "seed", "*", where)

    def get_snapshot_records(
        self, select: str = None, where: str = None  # type: ignore
    ) -> List[tuple]:
        return common.get_records(self.project, "snapshot", select, where)

    def delete_snapshot_records(self):
        common.delete_records(self.project, "snapshot")

    def _assert_results(
        self,
        ids_with_current_snapshot_records: Iterable,
        ids_with_closed_out_snapshot_records: Iterable,
    ):
        """
        All test cases are checked by considering whether a source record's id has a value in `dbt_valid_to`
        in `snapshot`. Each id can fall into one of the following cases:

        - The id has only one record in `snapshot`; it has a value in `dbt_valid_to`
            - the record was hard deleted in the source
        - The id has only one record in `snapshot`; it does not have a value in `dbt_valid_to`
            - the record was not updated in the source
            - the record was updated in the source, but not in a way that is tracked (e.g. via `strategy='check'`)
        - The id has two records in `snapshot`; one has a value in `dbt_valid_to`, the other does not
            - the record was altered in the source in a way that is tracked
            - the record was hard deleted and revived

        Note: Because of the third scenario, ids may show up in both arguments of this method.

        Args:
            ids_with_current_snapshot_records: a list/set/etc. of ids which aren't end-dated
            ids_with_closed_out_snapshot_records: a list/set/etc. of ids which are end-dated
        """
        records = set(self.get_snapshot_records("id, dbt_valid_to is null as is_current"))
        expected_records = set().union(
            {(i, True) for i in ids_with_current_snapshot_records},
            {(i, False) for i in ids_with_closed_out_snapshot_records},
        )
        for record in records:
            assert record in expected_records


class BaseSimpleSnapshot(BaseSimpleSnapshotBase):
    @pytest.fixture(scope="class")
    def snapshots(self):
        return {"snapshot.sql": snapshots.SNAPSHOT_TIMESTAMP_SQL}

    def test_updates_are_captured_by_snapshot(self, project):
        """
        Update the last 5 records. Show that all ids are current, but the last 5 reflect updates.
        """
        self.update_fact_records(
            {"updated_at": "updated_at + interval '1 day'"}, "id between 16 and 20"
        )
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 21),
            ids_with_closed_out_snapshot_records=range(16, 21),
        )

    def test_inserts_are_captured_by_snapshot(self, project):
        """
        Insert 10 records. Show that there are 30 records in `snapshot`, all of which are current.
        """
        self.insert_fact_records("id between 21 and 30")
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 31), ids_with_closed_out_snapshot_records=[]
        )

    def test_deletes_are_captured_by_snapshot(self, project):
        """
        Hard delete the last five records. Show that there are now only 15 current records and 5 expired records.
        """
        self.delete_fact_records("id between 16 and 20")
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 16),
            ids_with_closed_out_snapshot_records=range(16, 21),
        )

    def test_revives_are_captured_by_snapshot(self, project):
        """
        Delete the last five records and run snapshot to collect that information, then revive 3 of those records.
        Show that there are now 18 current records and 5 expired records.
        """
        self.delete_fact_records("id between 16 and 20")
        run_dbt(["snapshot"])
        self.insert_fact_records("id between 16 and 18")
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 19),
            ids_with_closed_out_snapshot_records=range(16, 21),
        )

    def test_new_column_captured_by_snapshot(self, project):
        """
        Add a column to `fact` and populate the last 10 records with a non-null value.
        Show that all ids are current, but the last 10 reflect updates and the first 10 don't
        i.e. if the column is added, but not updated, the record doesn't reflect that it's updated
        """
        self.add_fact_column("full_name", "varchar(200) default null")
        self.update_fact_records(
            {
                "full_name": "first_name || ' ' || last_name",
                "updated_at": "updated_at + interval '1 day'",
            },
            "id between 11 and 20",
        )
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 21),
            ids_with_closed_out_snapshot_records=range(11, 21),
        )


class BaseSnapshotCheck(BaseSimpleSnapshotBase):
    @pytest.fixture(scope="class")
    def snapshots(self):
        return {"snapshot.sql": snapshots.SNAPSHOT_CHECK_SQL}

    def test_column_selection_is_reflected_in_snapshot(self, project):
        """
        Update the first 10 records on a non-tracked column.
        Update the middle 10 records on a tracked column. (hence records 6-10 are updated on both)
        Show that all ids are current, and only the tracked column updates are reflected in `snapshot`.
        """
        self.update_fact_records(
            {"last_name": "left(last_name, 3)"}, "id between 1 and 10"
        )  # not tracked
        self.update_fact_records({"email": "left(email, 3)"}, "id between 6 and 15")  # tracked
        run_dbt(["snapshot"])
        self._assert_results(
            ids_with_current_snapshot_records=range(1, 21),
            ids_with_closed_out_snapshot_records=range(6, 16),
        )


class TestSnapshot(BaseSimpleSnapshot):
    pass


class TestSnapshotCheck(BaseSnapshotCheck):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_equals
from dbt.tests.util import relation_from_name, run_dbt


class BaseEquals:
    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "equals.sql": base_utils.macros__equals_sql,
        }

    @pytest.fixture(scope="class")
    def seeds(self):
        return {
            "data_equals.csv": fixture_equals.SEEDS__DATA_EQUALS_CSV,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "equal_values.sql": fixture_equals.MODELS__EQUAL_VALUES_SQL,
            "not_equal_values.sql": fixture_equals.MODELS__NOT_EQUAL_VALUES_SQL,
        }

    def test_equal_values(self, project):
        run_dbt(["seed"])
        run_dbt(["run"])

        # There are 9 cases total; 3 are equal and 6 are not equal

        # 3 are equal
        relation = relation_from_name(project.adapter, "equal_values")
        result = project.run_sql(
            f"select count(*) as num_rows from {relation} where expected = 'same'", fetch="one"
        )
        assert result[0] == 3

        # 6 are not equal
        relation = relation_from_name(project.adapter, "not_equal_values")
        result = project.run_sql(
            f"select count(*) as num_rows from {relation} where expected = 'different'",
            fetch="one",
        )
        assert result[0] == 6


class TestEquals(BaseEquals):
    pass



#
# Macros
#

macros__schema_test = """
{% test column_type(model, column_name, type) %}

    {% set cols = adapter.get_columns_in_relation(model) %}

    {% set col_types = {} %}
    {% for col in cols %}
        {% do col_types.update({col.name: col.data_type}) %}
    {% endfor %}

    {% set validation_message = 'Got a column type of ' ~ col_types.get(column_name) ~ ', expected ' ~ type %}

    {% set val = 0 if col_types.get(column_name) == type else 1 %}
    {% if val == 1 and execute %}
        {{ log(validation_message, info=True) }}
    {% endif %}

    select '{{ validation_message }}' as validation_error
    from (select true) as nothing
    where {{ val }} = 1

{% endtest %}

"""

#
# Models
#

models__downstream_from_seed_actual = """
select * from {{ ref('seed_actual') }}

"""
models__downstream_from_seed_pipe_separated = """
select * from {{ ref('seed_pipe_separated') }}

"""
models__from_basic_seed = """
select * from {{ this.schema }}.seed_expected

"""

#
# Properties
#

properties__schema_yml = """
version: 2
seeds:
- name: seed_enabled
  columns:
  - name: birthday
    data_tests:
    - column_type:
        type: date
  - name: seed_id
    data_tests:
    - column_type:
        type: text

- name: seed_tricky
  columns:
  - name: seed_id
    data_tests:
    - column_type:
        type: integer
  - name: seed_id_str
    data_tests:
    - column_type:
        type: text
  - name: a_bool
    data_tests:
    - column_type:
        type: boolean
  - name: looks_like_a_bool
    data_tests:
    - column_type:
        type: text
  - name: a_date
    data_tests:
    - column_type:
        type: timestamp without time zone
  - name: looks_like_a_date
    data_tests:
    - column_type:
        type: text
  - name: relative
    data_tests:
    - column_type:
        type: text
  - name: weekday
    data_tests:
    - column_type:
        type: text
"""



import pytest

from dbt.tests.adapter.dbt_show import fixtures
from dbt.tests.util import run_dbt


# -- Below we define base classes for tests you import based on if your adapter supports dbt show or not --
class BaseShowLimit:
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "sample_model.sql": fixtures.models__sample_model,
            "ephemeral_model.sql": fixtures.models__ephemeral_model,
        }

    @pytest.fixture(scope="class")
    def seeds(self):
        return {"sample_seed.csv": fixtures.seeds__sample_seed}

    @pytest.mark.parametrize(
        "args,expected",
        [
            ([], 5),  # default limit
            (["--limit", 3], 3),  # fetch 3 rows
            (["--limit", -1], 7),  # fetch all rows
        ],
    )
    def test_limit(self, project, args, expected):
        run_dbt(["build"])
        dbt_args = ["show", "--inline", fixtures.models__second_ephemeral_model, *args]
        results = run_dbt(dbt_args)
        assert len(results.results[0].agate_table) == expected
        # ensure limit was injected in compiled_code when limit specified in command args
        limit = results.args.get("limit")
        if limit > 0:
            assert f"limit {limit}" in results.results[0].node.compiled_code


class BaseShowSqlHeader:
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "sql_header.sql": fixtures.models__sql_header,
        }

    def test_sql_header(self, project):
        run_dbt(["show", "--select", "sql_header", "--vars", "timezone: Asia/Kolkata"])


class TestPostgresShowSqlHeader(BaseShowSqlHeader):
    pass


class TestPostgresShowLimit(BaseShowLimit):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_generate_series


class BaseGenerateSeries(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_generate_series.yml": fixture_generate_series.models__test_generate_series_yml,
            "test_generate_series.sql": self.interpolate_macro_namespace(
                fixture_generate_series.models__test_generate_series_sql, "generate_series"
            ),
        }


class TestGenerateSeries(BaseGenerateSeries):
    pass



MACROS__MACRO_SQL = """
{%- macro query_header_no_args() -%}
{%- set x = "are pretty cool" -%}
{{ "dbt macros" }}
{{ x }}
{%- endmacro -%}


{%- macro query_header_args(message) -%}
  {%- set comment_dict = dict(
    app='dbt++',
    macro_version='0.1.0',
    message='blah: '~ message) -%}
  {{ return(comment_dict) }}
{%- endmacro -%}


{%- macro ordered_to_json(dct) -%}
{{ tojson(dct, sort_keys=True) }}
{%- endmacro %}


{% macro invalid_query_header() -%}
{{ "Here is an invalid character for you: */" }}
{% endmacro %}

"""

MODELS__X_SQL = """
{% do run_query('select 2 as inner_id') %}
select 1 as outer_id
"""



# cast_bool_to_text

models__test_cast_bool_to_text_sql = """
with data as (

    select 0=1 as input, 'false' as expected union all
    select 1=1 as input, 'true' as expected union all
    select null as input, null as expected

)

select

    {{ cast_bool_to_text("input") }} as actual,
    expected

from data
"""


models__test_cast_bool_to_text_yml = """
version: 2
models:
  - name: test_cast_bool_to_text
    data_tests:
      - assert_equal:
          actual: actual
          expected: expected
"""



# position

seeds__data_position_csv = """substring_text,string_text,result
def,abcdef,4
land,earth,0
town,fishtown,5
ember,december,4
"""


models__test_position_sql = """
with data as (

    select * from {{ ref('data_position') }}

)

select

    {{ position('substring_text', 'string_text') }} as actual,
    result as expected

from data
"""


models__test_position_yml = """
version: 2
models:
  - name: test_position
    data_tests:
      - assert_equal:
          actual: actual
          expected: expected
"""



import pytest

from dbt.tests.util import (
    check_relations_equal,
    check_table_does_not_exist,
    rm_file,
    run_dbt,
    run_dbt_and_capture,
    write_file,
)


models__invalid_sql = """
{{
  config(
    materialized = "table"
  )
}}

select a_field_that_does_not_exist from {{ this.schema }}.seed

"""

models__table_a_sql = """
{{
  config(
    materialized = "table"
  )
}}

select * from {{ this.schema }}.seed

"""

models__table_b_sql = """
{{
  config(
    materialized = "table"
  )
}}

select * from {{ this.schema }}.seed

"""

models__view_model_sql = """
{{
  config(
    materialized = "view"
  )
}}

select * from {{ this.schema }}.seed

"""

models__dep_sql = """
{{
  config(
    materialized = "table"
  )
}}

select * from {{ref('view_model')}}

"""

models__view_with_conflicting_cascade_sql = """
select * from {{ref('table_a')}}

union all

select * from {{ref('table_b')}}

"""

models__skip_sql = """
select * from {{ref('invalid')}}

"""

seeds__seed_csv = """id,first_name,last_name,email,gender,ip_address
1,Jack,Hunter,jhunter0@pbs.org,Male,59.80.20.168
2,Kathryn,Walker,kwalker1@ezinearticles.com,Female,194.121.179.35
3,Gerald,Ryan,gryan2@com.com,Male,11.3.212.243
4,Bonnie,Spencer,bspencer3@ameblo.jp,Female,216.32.196.175
5,Harold,Taylor,htaylor4@people.com.cn,Male,253.10.246.136
6,Jacqueline,Griffin,jgriffin5@t.co,Female,16.13.192.220
7,Wanda,Arnold,warnold6@google.nl,Female,232.116.150.64
8,Craig,Ortiz,cortiz7@sciencedaily.com,Male,199.126.106.13
9,Gary,Day,gday8@nih.gov,Male,35.81.68.186
10,Rose,Wright,rwright9@yahoo.co.jp,Female,236.82.178.100
11,Raymond,Kelley,rkelleya@fc2.com,Male,213.65.166.67
12,Gerald,Robinson,grobinsonb@disqus.com,Male,72.232.194.193
13,Mildred,Martinez,mmartinezc@samsung.com,Female,198.29.112.5
14,Dennis,Arnold,darnoldd@google.com,Male,86.96.3.250
15,Judy,Gray,jgraye@opensource.org,Female,79.218.162.245
16,Theresa,Garza,tgarzaf@epa.gov,Female,21.59.100.54
17,Gerald,Robertson,grobertsong@csmonitor.com,Male,131.134.82.96
18,Philip,Hernandez,phernandezh@adobe.com,Male,254.196.137.72
19,Julia,Gonzalez,jgonzalezi@cam.ac.uk,Female,84.240.227.174
20,Andrew,Davis,adavisj@patch.com,Male,9.255.67.25
21,Kimberly,Harper,kharperk@foxnews.com,Female,198.208.120.253
22,Mark,Martin,mmartinl@marketwatch.com,Male,233.138.182.153
23,Cynthia,Ruiz,cruizm@google.fr,Female,18.178.187.201
24,Samuel,Carroll,scarrolln@youtu.be,Male,128.113.96.122
25,Jennifer,Larson,jlarsono@vinaora.com,Female,98.234.85.95
26,Ashley,Perry,aperryp@rakuten.co.jp,Female,247.173.114.52
27,Howard,Rodriguez,hrodriguezq@shutterfly.com,Male,231.188.95.26
28,Amy,Brooks,abrooksr@theatlantic.com,Female,141.199.174.118
29,Louise,Warren,lwarrens@adobe.com,Female,96.105.158.28
30,Tina,Watson,twatsont@myspace.com,Female,251.142.118.177
31,Janice,Kelley,jkelleyu@creativecommons.org,Female,239.167.34.233
32,Terry,Mccoy,tmccoyv@bravesites.com,Male,117.201.183.203
33,Jeffrey,Morgan,jmorganw@surveymonkey.com,Male,78.101.78.149
34,Louis,Harvey,lharveyx@sina.com.cn,Male,51.50.0.167
35,Philip,Miller,pmillery@samsung.com,Male,103.255.222.110
36,Willie,Marshall,wmarshallz@ow.ly,Male,149.219.91.68
37,Patrick,Lopez,plopez10@redcross.org,Male,250.136.229.89
38,Adam,Jenkins,ajenkins11@harvard.edu,Male,7.36.112.81
39,Benjamin,Cruz,bcruz12@linkedin.com,Male,32.38.98.15
40,Ruby,Hawkins,rhawkins13@gmpg.org,Female,135.171.129.255
41,Carlos,Barnes,cbarnes14@a8.net,Male,240.197.85.140
42,Ruby,Griffin,rgriffin15@bravesites.com,Female,19.29.135.24
43,Sean,Mason,smason16@icq.com,Male,159.219.155.249
44,Anthony,Payne,apayne17@utexas.edu,Male,235.168.199.218
45,Steve,Cruz,scruz18@pcworld.com,Male,238.201.81.198
46,Anthony,Garcia,agarcia19@flavors.me,Male,25.85.10.18
47,Doris,Lopez,dlopez1a@sphinn.com,Female,245.218.51.238
48,Susan,Nichols,snichols1b@freewebs.com,Female,199.99.9.61
49,Wanda,Ferguson,wferguson1c@yahoo.co.jp,Female,236.241.135.21
50,Andrea,Pierce,apierce1d@google.co.uk,Female,132.40.10.209
51,Lawrence,Phillips,lphillips1e@jugem.jp,Male,72.226.82.87
52,Judy,Gilbert,jgilbert1f@multiply.com,Female,196.250.15.142
53,Eric,Williams,ewilliams1g@joomla.org,Male,222.202.73.126
54,Ralph,Romero,rromero1h@sogou.com,Male,123.184.125.212
55,Jean,Wilson,jwilson1i@ocn.ne.jp,Female,176.106.32.194
56,Lori,Reynolds,lreynolds1j@illinois.edu,Female,114.181.203.22
57,Donald,Moreno,dmoreno1k@bbc.co.uk,Male,233.249.97.60
58,Steven,Berry,sberry1l@eepurl.com,Male,186.193.50.50
59,Theresa,Shaw,tshaw1m@people.com.cn,Female,120.37.71.222
60,John,Stephens,jstephens1n@nationalgeographic.com,Male,191.87.127.115
61,Richard,Jacobs,rjacobs1o@state.tx.us,Male,66.210.83.155
62,Andrew,Lawson,alawson1p@over-blog.com,Male,54.98.36.94
63,Peter,Morgan,pmorgan1q@rambler.ru,Male,14.77.29.106
64,Nicole,Garrett,ngarrett1r@zimbio.com,Female,21.127.74.68
65,Joshua,Kim,jkim1s@edublogs.org,Male,57.255.207.41
66,Ralph,Roberts,rroberts1t@people.com.cn,Male,222.143.131.109
67,George,Montgomery,gmontgomery1u@smugmug.com,Male,76.75.111.77
68,Gerald,Alvarez,galvarez1v@flavors.me,Male,58.157.186.194
69,Donald,Olson,dolson1w@whitehouse.gov,Male,69.65.74.135
70,Carlos,Morgan,cmorgan1x@pbs.org,Male,96.20.140.87
71,Aaron,Stanley,astanley1y@webnode.com,Male,163.119.217.44
72,Virginia,Long,vlong1z@spiegel.de,Female,204.150.194.182
73,Robert,Berry,rberry20@tripadvisor.com,Male,104.19.48.241
74,Antonio,Brooks,abrooks21@unesco.org,Male,210.31.7.24
75,Ruby,Garcia,rgarcia22@ovh.net,Female,233.218.162.214
76,Jack,Hanson,jhanson23@blogtalkradio.com,Male,31.55.46.199
77,Kathryn,Nelson,knelson24@walmart.com,Female,14.189.146.41
78,Jason,Reed,jreed25@printfriendly.com,Male,141.189.89.255
79,George,Coleman,gcoleman26@people.com.cn,Male,81.189.221.144
80,Rose,King,rking27@ucoz.com,Female,212.123.168.231
81,Johnny,Holmes,jholmes28@boston.com,Male,177.3.93.188
82,Katherine,Gilbert,kgilbert29@altervista.org,Female,199.215.169.61
83,Joshua,Thomas,jthomas2a@ustream.tv,Male,0.8.205.30
84,Julie,Perry,jperry2b@opensource.org,Female,60.116.114.192
85,Richard,Perry,rperry2c@oracle.com,Male,181.125.70.232
86,Kenneth,Ruiz,kruiz2d@wikimedia.org,Male,189.105.137.109
87,Jose,Morgan,jmorgan2e@webnode.com,Male,101.134.215.156
88,Donald,Campbell,dcampbell2f@goo.ne.jp,Male,102.120.215.84
89,Debra,Collins,dcollins2g@uol.com.br,Female,90.13.153.235
90,Jesse,Johnson,jjohnson2h@stumbleupon.com,Male,225.178.125.53
91,Elizabeth,Stone,estone2i@histats.com,Female,123.184.126.221
92,Angela,Rogers,arogers2j@goodreads.com,Female,98.104.132.187
93,Emily,Dixon,edixon2k@mlb.com,Female,39.190.75.57
94,Albert,Scott,ascott2l@tinypic.com,Male,40.209.13.189
95,Barbara,Peterson,bpeterson2m@ow.ly,Female,75.249.136.180
96,Adam,Greene,agreene2n@fastcompany.com,Male,184.173.109.144
97,Earl,Sanders,esanders2o@hc360.com,Male,247.34.90.117
98,Angela,Brooks,abrooks2p@mtv.com,Female,10.63.249.126
99,Harold,Foster,hfoster2q@privacy.gov.au,Male,139.214.40.244
100,Carl,Meyer,cmeyer2r@disqus.com,Male,204.117.7.88
"""

seeds__update_csv = """id,first_name,last_name,email,gender,ip_address
1,Michael,Perez,mperez0@chronoengine.com,Male,106.239.70.175
2,Shawn,Mccoy,smccoy1@reddit.com,Male,24.165.76.182
3,Kathleen,Payne,kpayne2@cargocollective.com,Female,113.207.168.106
4,Jimmy,Cooper,jcooper3@cargocollective.com,Male,198.24.63.114
5,Katherine,Rice,krice4@typepad.com,Female,36.97.186.238
6,Sarah,Ryan,sryan5@gnu.org,Female,119.117.152.40
7,Martin,Mcdonald,mmcdonald6@opera.com,Male,8.76.38.115
8,Frank,Robinson,frobinson7@wunderground.com,Male,186.14.64.194
9,Jennifer,Franklin,jfranklin8@mail.ru,Female,91.216.3.131
10,Henry,Welch,hwelch9@list-manage.com,Male,176.35.182.168
11,Fred,Snyder,fsnydera@reddit.com,Male,217.106.196.54
12,Amy,Dunn,adunnb@nba.com,Female,95.39.163.195
13,Kathleen,Meyer,kmeyerc@cdc.gov,Female,164.142.188.214
14,Steve,Ferguson,sfergusond@reverbnation.com,Male,138.22.204.251
15,Teresa,Hill,thille@dion.ne.jp,Female,82.84.228.235
16,Amanda,Harper,aharperf@mail.ru,Female,16.123.56.176
17,Kimberly,Ray,krayg@xing.com,Female,48.66.48.12
18,Johnny,Knight,jknighth@jalbum.net,Male,99.30.138.123
19,Virginia,Freeman,vfreemani@tiny.cc,Female,225.172.182.63
20,Anna,Austin,aaustinj@diigo.com,Female,62.111.227.148
21,Willie,Hill,whillk@mail.ru,Male,0.86.232.249
22,Sean,Harris,sharrisl@zdnet.com,Male,117.165.133.249
23,Mildred,Adams,madamsm@usatoday.com,Female,163.44.97.46
24,David,Graham,dgrahamn@zimbio.com,Male,78.13.246.202
25,Victor,Hunter,vhuntero@ehow.com,Male,64.156.179.139
26,Aaron,Ruiz,aruizp@weebly.com,Male,34.194.68.78
27,Benjamin,Brooks,bbrooksq@jalbum.net,Male,20.192.189.107
28,Lisa,Wilson,lwilsonr@japanpost.jp,Female,199.152.130.217
29,Benjamin,King,bkings@comsenz.com,Male,29.189.189.213
30,Christina,Williamson,cwilliamsont@boston.com,Female,194.101.52.60
31,Jane,Gonzalez,jgonzalezu@networksolutions.com,Female,109.119.12.87
32,Thomas,Owens,towensv@psu.edu,Male,84.168.213.153
33,Katherine,Moore,kmoorew@naver.com,Female,183.150.65.24
34,Jennifer,Stewart,jstewartx@yahoo.com,Female,38.41.244.58
35,Sara,Tucker,stuckery@topsy.com,Female,181.130.59.184
36,Harold,Ortiz,hortizz@vkontakte.ru,Male,198.231.63.137
37,Shirley,James,sjames10@yelp.com,Female,83.27.160.104
38,Dennis,Johnson,djohnson11@slate.com,Male,183.178.246.101
39,Louise,Weaver,lweaver12@china.com.cn,Female,1.14.110.18
40,Maria,Armstrong,marmstrong13@prweb.com,Female,181.142.1.249
41,Gloria,Cruz,gcruz14@odnoklassniki.ru,Female,178.232.140.243
42,Diana,Spencer,dspencer15@ifeng.com,Female,125.153.138.244
43,Kelly,Nguyen,knguyen16@altervista.org,Female,170.13.201.119
44,Jane,Rodriguez,jrodriguez17@biblegateway.com,Female,12.102.249.81
45,Scott,Brown,sbrown18@geocities.jp,Male,108.174.99.192
46,Norma,Cruz,ncruz19@si.edu,Female,201.112.156.197
47,Marie,Peters,mpeters1a@mlb.com,Female,231.121.197.144
48,Lillian,Carr,lcarr1b@typepad.com,Female,206.179.164.163
49,Judy,Nichols,jnichols1c@t-online.de,Female,158.190.209.194
50,Billy,Long,blong1d@yahoo.com,Male,175.20.23.160
51,Howard,Reid,hreid1e@exblog.jp,Male,118.99.196.20
52,Laura,Ferguson,lferguson1f@tuttocitta.it,Female,22.77.87.110
53,Anne,Bailey,abailey1g@geocities.com,Female,58.144.159.245
54,Rose,Morgan,rmorgan1h@ehow.com,Female,118.127.97.4
55,Nicholas,Reyes,nreyes1i@google.ru,Male,50.135.10.252
56,Joshua,Kennedy,jkennedy1j@house.gov,Male,154.6.163.209
57,Paul,Watkins,pwatkins1k@upenn.edu,Male,177.236.120.87
58,Kathryn,Kelly,kkelly1l@businessweek.com,Female,70.28.61.86
59,Adam,Armstrong,aarmstrong1m@techcrunch.com,Male,133.235.24.202
60,Norma,Wallace,nwallace1n@phoca.cz,Female,241.119.227.128
61,Timothy,Reyes,treyes1o@google.cn,Male,86.28.23.26
62,Elizabeth,Patterson,epatterson1p@sun.com,Female,139.97.159.149
63,Edward,Gomez,egomez1q@google.fr,Male,158.103.108.255
64,David,Cox,dcox1r@friendfeed.com,Male,206.80.80.58
65,Brenda,Wood,bwood1s@over-blog.com,Female,217.207.44.179
66,Adam,Walker,awalker1t@blogs.com,Male,253.211.54.93
67,Michael,Hart,mhart1u@wix.com,Male,230.206.200.22
68,Jesse,Ellis,jellis1v@google.co.uk,Male,213.254.162.52
69,Janet,Powell,jpowell1w@un.org,Female,27.192.194.86
70,Helen,Ford,hford1x@creativecommons.org,Female,52.160.102.168
71,Gerald,Carpenter,gcarpenter1y@about.me,Male,36.30.194.218
72,Kathryn,Oliver,koliver1z@army.mil,Female,202.63.103.69
73,Alan,Berry,aberry20@gov.uk,Male,246.157.112.211
74,Harry,Andrews,handrews21@ameblo.jp,Male,195.108.0.12
75,Andrea,Hall,ahall22@hp.com,Female,149.162.163.28
76,Barbara,Wells,bwells23@behance.net,Female,224.70.72.1
77,Anne,Wells,awells24@apache.org,Female,180.168.81.153
78,Harry,Harper,hharper25@rediff.com,Male,151.87.130.21
79,Jack,Ray,jray26@wufoo.com,Male,220.109.38.178
80,Phillip,Hamilton,phamilton27@joomla.org,Male,166.40.47.30
81,Shirley,Hunter,shunter28@newsvine.com,Female,97.209.140.194
82,Arthur,Daniels,adaniels29@reuters.com,Male,5.40.240.86
83,Virginia,Rodriguez,vrodriguez2a@walmart.com,Female,96.80.164.184
84,Christina,Ryan,cryan2b@hibu.com,Female,56.35.5.52
85,Theresa,Mendoza,tmendoza2c@vinaora.com,Female,243.42.0.210
86,Jason,Cole,jcole2d@ycombinator.com,Male,198.248.39.129
87,Phillip,Bryant,pbryant2e@rediff.com,Male,140.39.116.251
88,Adam,Torres,atorres2f@sun.com,Male,101.75.187.135
89,Margaret,Johnston,mjohnston2g@ucsd.edu,Female,159.30.69.149
90,Paul,Payne,ppayne2h@hhs.gov,Male,199.234.140.220
91,Todd,Willis,twillis2i@businessweek.com,Male,191.59.136.214
92,Willie,Oliver,woliver2j@noaa.gov,Male,44.212.35.197
93,Frances,Robertson,frobertson2k@go.com,Female,31.117.65.136
94,Gregory,Hawkins,ghawkins2l@joomla.org,Male,91.3.22.49
95,Lisa,Perkins,lperkins2m@si.edu,Female,145.95.31.186
96,Jacqueline,Anderson,janderson2n@cargocollective.com,Female,14.176.0.187
97,Shirley,Diaz,sdiaz2o@ucla.edu,Female,207.12.95.46
98,Nicole,Meyer,nmeyer2p@flickr.com,Female,231.79.115.13
99,Mary,Gray,mgray2q@constantcontact.com,Female,210.116.64.253
100,Jean,Mcdonald,jmcdonald2r@baidu.com,Female,122.239.235.117
"""


class BaseConcurrency:
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"seed.csv": seeds__seed_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "invalid.sql": models__invalid_sql,
            "table_a.sql": models__table_a_sql,
            "table_b.sql": models__table_b_sql,
            "view_model.sql": models__view_model_sql,
            "dep.sql": models__dep_sql,
            "view_with_conflicting_cascade.sql": models__view_with_conflicting_cascade_sql,
            "skip.sql": models__skip_sql,
        }

    def test_concurrency(self, project):
        run_dbt(["seed", "--select", "seed"])
        results = run_dbt(["run"], expect_pass=False)
        assert len(results) == 7
        check_relations_equal(project.adapter, ["seed", "view_model"])
        check_relations_equal(project.adapter, ["seed", "dep"])
        check_relations_equal(project.adapter, ["seed", "table_a"])
        check_relations_equal(project.adapter, ["seed", "table_b"])
        check_table_does_not_exist(project.adapter, "invalid")
        check_table_does_not_exist(project.adapter, "skip")

        rm_file(project.project_root, "seeds", "seed.csv")
        write_file(seeds__update_csv, project.project_root, "seeds", "seed.csv")

        results, output = run_dbt_and_capture(["run"], expect_pass=False)
        assert len(results) == 7
        check_relations_equal(project.adapter, ["seed", "view_model"])
        check_relations_equal(project.adapter, ["seed", "dep"])
        check_relations_equal(project.adapter, ["seed", "table_a"])
        check_relations_equal(project.adapter, ["seed", "table_b"])
        check_table_does_not_exist(project.adapter, "invalid")
        check_table_does_not_exist(project.adapter, "skip")

        assert "PASS=5 WARN=0 ERROR=1 SKIP=1 TOTAL=7" in output


class TestConcurenncy(BaseConcurrency):
    pass



import os
from pathlib import Path

import pytest

from dbt.tests.adapter.hooks import fixtures
from dbt.tests.util import check_table_does_not_exist, run_dbt


class BasePrePostRunHooks:
    @pytest.fixture(scope="function")
    def setUp(self, project):
        project.run_sql_file(project.test_data_dir / Path("seed_run.sql"))
        project.run_sql(f"drop table if exists { project.test_schema }.schemas")
        project.run_sql(f"drop table if exists { project.test_schema }.db_schemas")
        os.environ["TERM_TEST"] = "TESTING"

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "hook.sql": fixtures.macros__hook,
            "before-and-after.sql": fixtures.macros__before_and_after,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {"hooks.sql": fixtures.models__hooks}

    @pytest.fixture(scope="class")
    def seeds(self):
        return {"example_seed.csv": fixtures.seeds__example_seed_csv}

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            # The create and drop table statements here validate that these hooks run
            # in the same order that they are defined. Drop before create is an error.
            # Also check that the table does not exist below.
            "on-run-start": [
                "{{ custom_run_hook('start', target, run_started_at, invocation_id) }}",
                "create table {{ target.schema }}.start_hook_order_test ( id int )",
                "drop table {{ target.schema }}.start_hook_order_test",
                "{{ log(env_var('TERM_TEST'), info=True) }}",
            ],
            "on-run-end": [
                "{{ custom_run_hook('end', target, run_started_at, invocation_id) }}",
                "create table {{ target.schema }}.end_hook_order_test ( id int )",
                "drop table {{ target.schema }}.end_hook_order_test",
                "create table {{ target.schema }}.schemas ( schema text )",
                "insert into {{ target.schema }}.schemas (schema) values {% for schema in schemas %}( '{{ schema }}' ){% if not loop.last %},{% endif %}{% endfor %}",
                "create table {{ target.schema }}.db_schemas ( db text, schema text )",
                "insert into {{ target.schema }}.db_schemas (db, schema) values {% for db, schema in database_schemas %}('{{ db }}', '{{ schema }}' ){% if not loop.last %},{% endif %}{% endfor %}",
            ],
            "seeds": {
                "quote_columns": False,
            },
        }

    def get_ctx_vars(self, state, project):
        fields = [
            "test_state",
            "target_dbname",
            "target_host",
            "target_name",
            "target_schema",
            "target_threads",
            "target_type",
            "target_user",
            "target_pass",
            "run_started_at",
            "invocation_id",
            "thread_id",
        ]
        field_list = ", ".join(['"{}"'.format(f) for f in fields])
        query = f"select {field_list} from {project.test_schema}.on_run_hook where test_state = '{state}'"

        vals = project.run_sql(query, fetch="all")
        assert len(vals) != 0, "nothing inserted into on_run_hook table"
        assert len(vals) == 1, "too many rows in hooks table"
        ctx = dict([(k, v) for (k, v) in zip(fields, vals[0])])

        return ctx

    def assert_used_schemas(self, project):
        schemas_query = "select * from {}.schemas".format(project.test_schema)
        results = project.run_sql(schemas_query, fetch="all")
        assert len(results) == 1
        assert results[0][0] == project.test_schema

        db_schemas_query = "select * from {}.db_schemas".format(project.test_schema)
        results = project.run_sql(db_schemas_query, fetch="all")
        assert len(results) == 1
        assert results[0][0] == project.database
        assert results[0][1] == project.test_schema

    def check_hooks(self, state, project, host):
        ctx = self.get_ctx_vars(state, project)

        assert ctx["test_state"] == state
        assert ctx["target_dbname"] == "dbt"
        assert ctx["target_host"] == host
        assert ctx["target_name"] == "default"
        assert ctx["target_schema"] == project.test_schema
        assert ctx["target_threads"] == 4
        assert ctx["target_type"] == "postgres"
        assert ctx["target_user"] == "root"
        assert ctx["target_pass"] == ""

        assert (
            ctx["run_started_at"] is not None and len(ctx["run_started_at"]) > 0
        ), "run_started_at was not set"
        assert (
            ctx["invocation_id"] is not None and len(ctx["invocation_id"]) > 0
        ), "invocation_id was not set"
        assert ctx["thread_id"].startswith("Thread-") or ctx["thread_id"] == "MainThread"

    def test_pre_and_post_run_hooks(self, setUp, project, dbt_profile_target):
        run_dbt(["run"])

        self.check_hooks("start", project, dbt_profile_target.get("host", None))
        self.check_hooks("end", project, dbt_profile_target.get("host", None))

        check_table_does_not_exist(project.adapter, "start_hook_order_test")
        check_table_does_not_exist(project.adapter, "end_hook_order_test")
        self.assert_used_schemas(project)

    def test_pre_and_post_seed_hooks(self, setUp, project, dbt_profile_target):
        run_dbt(["seed"])

        self.check_hooks("start", project, dbt_profile_target.get("host", None))
        self.check_hooks("end", project, dbt_profile_target.get("host", None))

        check_table_does_not_exist(project.adapter, "start_hook_order_test")
        check_table_does_not_exist(project.adapter, "end_hook_order_test")
        self.assert_used_schemas(project)


class TestPrePostRunHooks(BasePrePostRunHooks):
    pass


class BaseAfterRunHooks:
    @pytest.fixture(scope="class")
    def macros(self):
        return {"temp_macro.sql": fixtures.macros_missing_column}

    @pytest.fixture(scope="class")
    def models(self):
        return {"test_column.sql": fixtures.models__missing_column}

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            # The create and drop table statements here validate that these hooks run
            # in the same order that they are defined. Drop before create is an error.
            # Also check that the table does not exist below.
            "on-run-start": "- {{ export_table_check() }}"
        }

    def test_missing_column_pre_hook(self, project):
        run_dbt(["run"], expect_pass=False)


class TestAfterRunHooks(BaseAfterRunHooks):
    pass



import pytest

# TODO: update to new dbt-artifacts once available
from dbt.artifacts.schemas.results import RunStatus
from dbt.tests.adapter.basic import files
from dbt.tests.util import run_dbt, check_relations_equal, relation_from_name


class BaseIncremental:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"name": "incremental"}

    @pytest.fixture(scope="class")
    def models(self):
        return {"incremental.sql": files.incremental_sql, "schema.yml": files.schema_base_yml}

    @pytest.fixture(scope="class")
    def seeds(self):
        return {"base.csv": files.seeds_base_csv, "added.csv": files.seeds_added_csv}

    @pytest.fixture(autouse=True)
    def clean_up(self, project):
        yield
        with project.adapter.connection_named("__test"):
            relation = project.adapter.Relation.create(
                database=project.database, schema=project.test_schema
            )
            project.adapter.drop_schema(relation)

    pass

    def test_incremental(self, project):
        # seed command
        results = run_dbt(["seed"])
        assert len(results) == 2

        # base table rowcount
        relation = relation_from_name(project.adapter, "base")
        result = project.run_sql(f"select count(*) as num_rows from {relation}", fetch="one")
        assert result[0] == 10

        # added table rowcount
        relation = relation_from_name(project.adapter, "added")
        result = project.run_sql(f"select count(*) as num_rows from {relation}", fetch="one")
        assert result[0] == 20

        # run command
        # the "seed_name" var changes the seed identifier in the schema file
        results = run_dbt(["run", "--vars", "seed_name: base"])
        assert len(results) == 1

        # check relations equal
        check_relations_equal(project.adapter, ["base", "incremental"])

        # change seed_name var
        # the "seed_name" var changes the seed identifier in the schema file
        results = run_dbt(["run", "--vars", "seed_name: added"])
        assert len(results) == 1

        # check relations equal
        check_relations_equal(project.adapter, ["added", "incremental"])

        # get catalog from docs generate
        catalog = run_dbt(["docs", "generate"])
        assert len(catalog.nodes) == 3
        assert len(catalog.sources) == 1


class BaseIncrementalNotSchemaChange:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"name": "incremental"}

    @pytest.fixture(scope="class")
    def models(self):
        return {"incremental_not_schema_change.sql": files.incremental_not_schema_change_sql}

    def test_incremental_not_schema_change(self, project):
        # Schema change is not evaluated on first run, so two are needed
        run_dbt(["run", "--select", "incremental_not_schema_change"])
        run_result = (
            run_dbt(["run", "--select", "incremental_not_schema_change"]).results[0].status
        )

        assert run_result == RunStatus.Success


class Testincremental(BaseIncremental):
    pass


class TestBaseIncrementalNotSchemaChange(BaseIncrementalNotSchemaChange):
    pass



# macros #
MACROS__CAST_SQL = """


{% macro string_literal(s) -%}
  {{ adapter.dispatch('string_literal', macro_namespace='test')(s) }}
{%- endmacro %}

{% macro default__string_literal(s) %}
    '{{ s }}'::text
{% endmacro %}

"""

MACROS__EXPECT_VALUE_SQL = """

-- cross-db compatible test, similar to accepted_values

{% test expect_value(model, field, value) %}

select *
from {{ model }}
where {{ field }} != '{{ value }}'

{% endtest %}

"""

# base aliases #
MODELS__SCHEMA_YML = """
version: 2
models:
- name: foo_alias
  data_tests:
  - expect_value:
      field: tablename
      value: foo
- name: ref_foo_alias
  data_tests:
  - expect_value:
      field: tablename
      value: ref_foo_alias
- name: alias_in_project
  data_tests:
  - expect_value:
      field: tablename
      value: project_alias
- name: alias_in_project_with_override
  data_tests:
  - expect_value:
      field: tablename
      value: override_alias

"""

MODELS__FOO_ALIAS_SQL = """

{{
    config(
        alias='foo',
        materialized='table'
    )
}}

select {{ string_literal(this.name) }} as tablename

"""

MODELS__ALIAS_IN_PROJECT_SQL = """

select {{ string_literal(this.name) }} as tablename

"""

MODELS__ALIAS_IN_PROJECT_WITH_OVERRIDE_SQL = """

{{ config(alias='override_alias') }}

select {{ string_literal(this.name) }} as tablename

"""

MODELS__REF_FOO_ALIAS_SQL = """

{{
    config(
        materialized='table'
    )
}}

with trigger_ref as (

  -- we should still be able to ref a model by its filepath
  select * from {{ ref('foo_alias') }}

)

-- this name should still be the filename
select {{ string_literal(this.name) }} as tablename

"""

# error #
MODELS_DUPE__MODEL_A_SQL = """

{{ config(alias='duped_alias') }}

select 1 as id

"""

MODELS_DUPE__MODEL_B_SQL = """

{{ config(alias='duped_alias') }}

select 1 as id

"""

MODELS_DUPE__README_MD = """
these should fail because both models have the same alias
and are configured to build in the same schema

"""

# dupe custom database #
MODELS_DUPE_CUSTOM_DATABASE__SCHEMA_YML = """
version: 2
models:
- name: model_a
  data_tests:
  - expect_value:
      field: tablename
      value: duped_alias
- name: model_b
  data_tests:
  - expect_value:
      field: tablename
      value: duped_alias

"""

MODELS_DUPE_CUSTOM_DATABASE__MODEL_A_SQL = """
select {{ string_literal(this.name) }} as tablename

"""

MODELS_DUPE_CUSTOM_DATABASE__MODEL_B_SQL = """
select {{ string_literal(this.name) }} as tablename

"""

MODELS_DUPE_CUSTOM_DATABASE__README_MD = """
these should succeed, as both models have the same alias,
but they are configured to be built in _different_ schemas

"""

# dupe custom schema #
MODELS_DUPE_CUSTOM_SCHEMA__SCHEMA_YML = """
version: 2
models:
- name: model_a
  data_tests:
  - expect_value:
      field: tablename
      value: duped_alias
- name: model_b
  data_tests:
  - expect_value:
      field: tablename
      value: duped_alias
- name: model_c
  data_tests:
  - expect_value:
      field: tablename
      value: duped_alias

"""

MODELS_DUPE_CUSTOM_SCHEMA__MODEL_A_SQL = """

{{ config(alias='duped_alias', schema='schema_a') }}

select {{ string_literal(this.name) }} as tablename

"""

MODELS_DUPE_CUSTOM_SCHEMA__MODEL_B_SQL = """

{{ config(alias='duped_alias', schema='schema_b') }}

select {{ string_literal(this.name) }} as tablename

"""

MODELS_DUPE_CUSTOM_SCHEMA__MODEL_C_SQL = """

-- no custom schema for this model
{{ config(alias='duped_alias') }}

select {{ string_literal(this.name) }} as tablename

"""

MODELS_DUPE_CUSTOM_SCHEMA__README_MD = """
these should succeed, as both models have the same alias,
but they are configured to be built in _different_ schemas

"""



from collections import namedtuple

import pytest

from dbt.tests.util import check_relations_equal, run_dbt


models__delete_insert_incremental_predicates_sql = """
{{ config(
    materialized = 'incremental',
    unique_key = 'id'
) }}

{% if not is_incremental() %}

select 1 as id, 'hello' as msg, 'blue' as color
union all
select 2 as id, 'goodbye' as msg, 'red' as color

{% else %}

-- delete will not happen on the above record where id = 2, so new record will be inserted instead
select 1 as id, 'hey' as msg, 'blue' as color
union all
select 2 as id, 'yo' as msg, 'green' as color
union all
select 3 as id, 'anyway' as msg, 'purple' as color

{% endif %}
"""

seeds__expected_delete_insert_incremental_predicates_csv = """id,msg,color
1,hey,blue
2,goodbye,red
2,yo,green
3,anyway,purple
"""

ResultHolder = namedtuple(
    "ResultHolder",
    [
        "seed_count",
        "model_count",
        "seed_rows",
        "inc_test_model_count",
        "opt_model_count",
        "relation",
    ],
)


class BaseIncrementalPredicates:
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "delete_insert_incremental_predicates.sql": models__delete_insert_incremental_predicates_sql
        }

    @pytest.fixture(scope="class")
    def seeds(self):
        return {
            "expected_delete_insert_incremental_predicates.csv": seeds__expected_delete_insert_incremental_predicates_csv
        }

    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "models": {
                "+incremental_predicates": ["id != 2"],
                "+incremental_strategy": "delete+insert",
            }
        }

    def update_incremental_model(self, incremental_model):
        """update incremental model after the seed table has been updated"""
        model_result_set = run_dbt(["run", "--select", incremental_model])
        return len(model_result_set)

    def get_test_fields(
        self, project, seed, incremental_model, update_sql_file, opt_model_count=None
    ):
        seed_count = len(run_dbt(["seed", "--select", seed, "--full-refresh"]))

        model_count = len(run_dbt(["run", "--select", incremental_model, "--full-refresh"]))
        # pass on kwarg
        relation = incremental_model
        # update seed in anticipation of incremental model update
        row_count_query = "select * from {}.{}".format(project.test_schema, seed)
        # project.run_sql_file(Path("seeds") / Path(update_sql_file + ".sql"))
        seed_rows = len(project.run_sql(row_count_query, fetch="all"))

        # propagate seed state to incremental model according to unique keys
        inc_test_model_count = self.update_incremental_model(incremental_model=incremental_model)

        return ResultHolder(
            seed_count, model_count, seed_rows, inc_test_model_count, opt_model_count, relation
        )

    def check_scenario_correctness(self, expected_fields, test_case_fields, project):
        """Invoke assertions to verify correct build functionality"""
        # 1. test seed(s) should build afresh
        assert expected_fields.seed_count == test_case_fields.seed_count
        # 2. test model(s) should build afresh
        assert expected_fields.model_count == test_case_fields.model_count
        # 3. seeds should have intended row counts post update
        assert expected_fields.seed_rows == test_case_fields.seed_rows
        # 4. incremental test model(s) should be updated
        assert expected_fields.inc_test_model_count == test_case_fields.inc_test_model_count
        # 5. extra incremental model(s) should be built; optional since
        #   comparison may be between an incremental model and seed
        if expected_fields.opt_model_count and test_case_fields.opt_model_count:
            assert expected_fields.opt_model_count == test_case_fields.opt_model_count
        # 6. result table should match intended result set (itself a relation)
        check_relations_equal(
            project.adapter, [expected_fields.relation, test_case_fields.relation]
        )

    def get_expected_fields(self, relation, seed_rows, opt_model_count=None):
        return ResultHolder(
            seed_count=1,
            model_count=1,
            inc_test_model_count=1,
            seed_rows=seed_rows,
            opt_model_count=opt_model_count,
            relation=relation,
        )

    # no unique_key test
    def test__incremental_predicates(self, project):
        """seed should match model after two incremental runs"""

        expected_fields = self.get_expected_fields(
            relation="expected_delete_insert_incremental_predicates", seed_rows=4
        )
        test_case_fields = self.get_test_fields(
            project,
            seed="expected_delete_insert_incremental_predicates",
            incremental_model="delete_insert_incremental_predicates",
            update_sql_file=None,
        )
        self.check_scenario_correctness(expected_fields, test_case_fields, project)


class TestIncrementalPredicatesDeleteInsert(BaseIncrementalPredicates):
    pass


class TestPredicatesDeleteInsert(BaseIncrementalPredicates):
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {"models": {"+predicates": ["id != 2"], "+incremental_strategy": "delete+insert"}}



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_right


class BaseRight(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_right.csv": fixture_right.seeds__data_right_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_right.yml": fixture_right.models__test_right_yml,
            "test_right.sql": self.interpolate_macro_namespace(
                fixture_right.models__test_right_sql, "right"
            ),
        }


class TestRight(BaseRight):
    pass



import pytest

from dbt.tests.adapter.simple_copy import fixtures
from dbt.tests.util import run_dbt, check_relations_equal


class BaseSimpleCopyUppercase:
    @pytest.fixture(scope="class")
    def dbt_profile_target(self):
        return {
            "type": "postgres",
            "threads": 4,
            "host": "localhost",
            "port": 5432,
            "user": "root",
            "pass": "password",
            "dbname": "dbtMixedCase",
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "ADVANCED_INCREMENTAL.sql": fixtures._MODELS__ADVANCED_INCREMENTAL,
            "COMPOUND_SORT.sql": fixtures._MODELS__COMPOUND_SORT,
            "DISABLED.sql": fixtures._MODELS__DISABLED,
            "EMPTY.sql": fixtures._MODELS__EMPTY,
            "GET_AND_REF.sql": fixtures._MODELS_GET_AND_REF_UPPERCASE,
            "INCREMENTAL.sql": fixtures._MODELS__INCREMENTAL,
            "INTERLEAVED_SORT.sql": fixtures._MODELS__INTERLEAVED_SORT,
            "MATERIALIZED.sql": fixtures._MODELS__MATERIALIZED,
            "VIEW_MODEL.sql": fixtures._MODELS__VIEW_MODEL,
        }

    @pytest.fixture(scope="class")
    def properties(self):
        return {
            "schema.yml": fixtures._PROPERTIES__SCHEMA_YML,
        }

    @pytest.fixture(scope="class")
    def seeds(self):
        return {"seed.csv": fixtures._SEEDS__SEED_INITIAL}

    def test_simple_copy_uppercase(self, project):
        # Load the seed file and check that it worked
        results = run_dbt(["seed"])
        assert len(results) == 1

        # Run the project and ensure that all the models loaded
        results = run_dbt()
        assert len(results) == 7

        check_relations_equal(
            project.adapter, ["seed", "VIEW_MODEL", "INCREMENTAL", "MATERIALIZED", "GET_AND_REF"]
        )


class TestSimpleCopyUppercase(BaseSimpleCopyUppercase):
    pass



import pytest

from dbt.tests.util import relation_from_name, run_dbt


model_input_sql = """
select 1 as id
"""

ephemeral_model_input_sql = """
{{ config(materialized='ephemeral') }}
select 2 as id
"""

raw_source_csv = """id
3
"""


model_sql = """
select *
from {{ ref('model_input') }}
union all
select *
from {{ ref('ephemeral_model_input') }}
union all
select *
from {{ source('seed_sources', 'raw_source') }}
"""


schema_sources_yml = """
sources:
  - name: seed_sources
    schema: "{{ target.schema }}"
    tables:
      - name: raw_source
"""


class BaseTestEmpty:
    @pytest.fixture(scope="class")
    def seeds(self):
        return {
            "raw_source.csv": raw_source_csv,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "model_input.sql": model_input_sql,
            "ephemeral_model_input.sql": ephemeral_model_input_sql,
            "model.sql": model_sql,
            "sources.yml": schema_sources_yml,
        }

    def assert_row_count(self, project, relation_name: str, expected_row_count: int):
        relation = relation_from_name(project.adapter, relation_name)
        result = project.run_sql(f"select count(*) as num_rows from {relation}", fetch="one")
        assert result[0] == expected_row_count

    def test_run_with_empty(self, project):
        # create source from seed
        run_dbt(["seed"])

        # run without empty - 3 expected rows in output - 1 from each input
        run_dbt(["run"])
        self.assert_row_count(project, "model", 3)

        # run with empty - 0 expected rows in output
        run_dbt(["run", "--empty"])
        self.assert_row_count(project, "model", 0)


class BaseTestEmptyInlineSourceRef(BaseTestEmpty):
    @pytest.fixture(scope="class")
    def models(self):
        model_sql = """
            select * from {{ source('seed_sources', 'raw_source') }} as raw_source
            """

        return {
            "model.sql": model_sql,
            "sources.yml": schema_sources_yml,
        }

    def test_run_with_empty(self, project):
        # create source from seed
        run_dbt(["seed"])
        run_dbt(["run", "--empty", "--debug"])
        self.assert_row_count(project, "model", 0)


class TestEmpty(BaseTestEmpty):
    pass



import os

from dbt.tests.util import run_dbt


class BaseEmpty:
    def test_empty(self, project):
        # check seed
        results = run_dbt(["seed"])
        assert len(results) == 0
        run_results_path = os.path.join(project.project_root, "target", "run_results.json")
        assert os.path.exists(run_results_path)

        # check run
        results = run_dbt(["run"])
        assert len(results) == 0

        catalog_path = os.path.join(project.project_root, "target", "catalog.json")
        assert not os.path.exists(catalog_path)

        # check catalog
        catalog = run_dbt(["docs", "generate"])
        assert os.path.exists(run_results_path)
        assert os.path.exists(catalog_path)
        assert len(catalog.nodes) == 0
        assert len(catalog.sources) == 0


class TestEmpty(BaseEmpty):
    pass



seed_csv = """id,name
1,Alice
2,Bob
"""

table_model_sql = """
{{ config(materialized='table') }}
select * from {{ ref('ephemeral_model') }}

-- establish a macro dependency to trigger state:modified.macros
-- depends on: {{ my_macro() }}
"""

view_model_sql = """
select * from {{ ref('seed') }}

-- establish a macro dependency that trips infinite recursion if not handled
-- depends on: {{ my_infinitely_recursive_macro() }}
"""

ephemeral_model_sql = """
{{ config(materialized='ephemeral') }}
select * from {{ ref('view_model') }}
"""

exposures_yml = """
version: 2
exposures:
  - name: my_exposure
    type: application
    depends_on:
      - ref('view_model')
    owner:
      email: test@example.com
"""

schema_yml = """
version: 2
models:
  - name: view_model
    columns:
      - name: id
        data_tests:
          - unique:
              severity: error
          - not_null
      - name: name
"""

get_schema_name_sql = """
{% macro generate_schema_name(custom_schema_name, node) -%}
    {%- set default_schema = target.schema -%}
    {%- if custom_schema_name is not none -%}
        {{ return(default_schema ~ '_' ~ custom_schema_name|trim) }}
    -- put seeds into a separate schema in "prod", to verify that cloning in "dev" still works
    {%- elif target.name == 'default' and node.resource_type == 'seed' -%}
        {{ return(default_schema ~ '_' ~ 'seeds') }}
    {%- else -%}
        {{ return(default_schema) }}
    {%- endif -%}
{%- endmacro %}
"""

snapshot_sql = """
{% snapshot my_cool_snapshot %}

    {{
        config(
            target_database=database,
            target_schema=schema,
            unique_key='id',
            strategy='check',
            check_cols=['id'],
        )
    }}
    select * from {{ ref('view_model') }}

{% endsnapshot %}
"""
macros_sql = """
{% macro my_macro() %}
    {% do log('in a macro' ) %}
{% endmacro %}
"""

infinite_macros_sql = """
{# trigger infinite recursion if not handled #}

{% macro my_infinitely_recursive_macro() %}
  {{ return(adapter.dispatch('my_infinitely_recursive_macro')()) }}
{% endmacro %}

{% macro default__my_infinitely_recursive_macro() %}
    {% if unmet_condition %}
        {{ my_infinitely_recursive_macro() }}
    {% else %}
        {{ return('') }}
    {% endif %}
{% endmacro %}
"""

custom_can_clone_tables_false_macros_sql = """
{% macro can_clone_table() %}
    {{ return(False) }}
{% endmacro %}
"""



import pytest

from dbt.tests.util import run_dbt


model_sql = """
{{
    config(
        materialized='table'
    )
}}
select 1 as id
"""

another_schema_model_sql = """
{{
    config(
        materialized='table',
        schema='another_schema'
    )
}}
select 1 as id
"""


class BaseCachingTest:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "config-version": 2,
            "quoting": {
                "identifier": False,
                "schema": False,
            },
        }

    def run_and_inspect_cache(self, project, run_args=None):
        run_dbt(run_args)

        # the cache was empty at the start of the run.
        # the model materialization returned an unquoted relation and added to the cache.
        adapter = project.adapter
        assert len(adapter.cache.relations) == 1
        relation = list(adapter.cache.relations).pop()
        assert relation.schema == project.test_schema
        assert relation.schema == project.test_schema.lower()

        # on the second run, dbt will find a relation in the database during cache population.
        # this relation will be quoted, because list_relations_without_caching (by default) uses
        # quote_policy = {"database": True, "schema": True, "identifier": True}
        # when adding relations to the cache.
        run_dbt(run_args)
        adapter = project.adapter
        assert len(adapter.cache.relations) == 1
        second_relation = list(adapter.cache.relations).pop()

        # perform a case-insensitive + quote-insensitive comparison
        for key in ["database", "schema", "identifier"]:
            assert getattr(relation, key).lower() == getattr(second_relation, key).lower()

    def test_cache(self, project):
        self.run_and_inspect_cache(project, run_args=["run"])


class BaseCachingLowercaseModel(BaseCachingTest):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "model.sql": model_sql,
        }


class BaseCachingUppercaseModel(BaseCachingTest):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "MODEL.sql": model_sql,
        }


class BaseCachingSelectedSchemaOnly(BaseCachingTest):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "model.sql": model_sql,
            "another_schema_model.sql": another_schema_model_sql,
        }

    def test_cache(self, project):
        # this should only cache the schema containing the selected model
        run_args = ["--cache-selected-only", "run", "--select", "model"]
        self.run_and_inspect_cache(project, run_args)


class BaseNoPopulateCache(BaseCachingTest):
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "model.sql": model_sql,
        }

    def test_cache(self, project):
        # --no-populate-cache still allows the cache to populate all relations
        # under a schema, so the behavior here remains the same as other tests
        run_args = ["--no-populate-cache", "run"]
        self.run_and_inspect_cache(project, run_args)


class TestCachingLowerCaseModel(BaseCachingLowercaseModel):
    pass


class TestCachingUppercaseModel(BaseCachingUppercaseModel):
    pass


class TestCachingSelectedSchemaOnly(BaseCachingSelectedSchemaOnly):
    pass


class TestNoPopulateCache(BaseNoPopulateCache):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_length


class BaseLength(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_length.csv": fixture_length.seeds__data_length_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_length.yml": fixture_length.models__test_length_yml,
            "test_length.sql": self.interpolate_macro_namespace(
                fixture_length.models__test_length_sql, "length"
            ),
        }


class TestLength(BaseLength):
    pass



# dateadd

seeds__data_dateadd_csv = """from_time,interval_length,datepart,result
2018-01-01 01:00:00,1,day,2018-01-02 01:00:00
2018-01-01 01:00:00,1,month,2018-02-01 01:00:00
2018-01-01 01:00:00,1,year,2019-01-01 01:00:00
2018-01-01 01:00:00,1,hour,2018-01-01 02:00:00
,1,day,
"""


models__test_dateadd_sql = """
with data as (

    select * from {{ ref('data_dateadd') }}

)

select
    case
        when datepart = 'hour' then cast({{ dateadd('hour', 'interval_length', 'from_time') }} as {{ api.Column.translate_type('timestamp') }})
        when datepart = 'day' then cast({{ dateadd('day', 'interval_length', 'from_time') }} as {{ api.Column.translate_type('timestamp') }})
        when datepart = 'month' then cast({{ dateadd('month', 'interval_length', 'from_time') }} as {{ api.Column.translate_type('timestamp') }})
        when datepart = 'year' then cast({{ dateadd('year', 'interval_length', 'from_time') }} as {{ api.Column.translate_type('timestamp') }})
        else null
    end as actual,
    result as expected

from data
"""

models__test_dateadd_yml = """
version: 2
models:
  - name: test_dateadd
    data_tests:
      - assert_equal:
          actual: actual
          expected: expected
"""



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_replace


class BaseReplace(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_replace.csv": fixture_replace.seeds__data_replace_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_replace.yml": fixture_replace.models__test_replace_yml,
            "test_replace.sql": self.interpolate_macro_namespace(
                fixture_replace.models__test_replace_sql, "replace"
            ),
        }


class TestReplace(BaseReplace):
    pass



import pytest
from dbt.tests.util import run_dbt, run_dbt_and_capture


my_model_sql = """
select
    tested_column from {{ ref('my_upstream_model')}}
"""

my_upstream_model_sql = """
select 1 as tested_column
"""

test_my_model_yml = """
unit_tests:
  - name: test_invalid_input_column_name
    model: my_model
    given:
      - input: ref('my_upstream_model')
        rows:
          - {invalid_column_name: 1}
    expect:
      rows:
          - {tested_column: 1}
  - name: test_invalid_expect_column_name
    model: my_model
    given:
      - input: ref('my_upstream_model')
        rows:
          - {tested_column: 1}
    expect:
      rows:
          - {invalid_column_name: 1}
"""


class BaseUnitTestInvalidInput:
    @pytest.fixture(scope="class")
    def models(self):
        return {
            "my_model.sql": my_model_sql,
            "my_upstream_model.sql": my_upstream_model_sql,
            "unit_tests.yml": test_my_model_yml,
        }

    def test_invalid_input(self, project):
        results = run_dbt(["run"])
        assert len(results) == 2

        _, out = run_dbt_and_capture(
            ["test", "--select", "test_name:test_invalid_input_column_name"], expect_pass=False
        )
        assert (
            "Invalid column name: 'invalid_column_name' in unit test fixture for 'my_upstream_model'."
            in out
        )

        _, out = run_dbt_and_capture(
            ["test", "--select", "test_name:test_invalid_expect_column_name"], expect_pass=False
        )
        assert (
            "Invalid column name: 'invalid_column_name' in unit test fixture for expected output."
            in out
        )


class TestPostgresUnitTestInvalidInput(BaseUnitTestInvalidInput):
    pass



import pytest

from dbt.tests.adapter.utils import base_utils, fixture_cast


class BaseCast(base_utils.BaseUtils):
    @pytest.fixture(scope="class")
    def seeds(self):
        return {"data_cast.csv": fixture_cast.seeds__data_cast_csv}

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "test_cast.yml": fixture_cast.models__test_cast_yml,
            "test_cast.sql": self.interpolate_macro_namespace(
                self.interpolate_macro_namespace(fixture_cast.models__test_cast_sql, "cast"),
                "type_string",
            ),
        }


class TestCast(BaseCast):
    pass



import pytest

from dbt.tests.adapter.aliases import fixtures
from dbt.tests.util import run_dbt


class BaseAliases:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "config-version": 2,
            "macro-paths": ["macros"],
            "models": {
                "test": {
                    "alias_in_project": {
                        "alias": "project_alias",
                    },
                    "alias_in_project_with_override": {
                        "alias": "project_alias",
                    },
                }
            },
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "schema.yml": fixtures.MODELS__SCHEMA_YML,
            "foo_alias.sql": fixtures.MODELS__FOO_ALIAS_SQL,
            "alias_in_project.sql": fixtures.MODELS__ALIAS_IN_PROJECT_SQL,
            "alias_in_project_with_override.sql": fixtures.MODELS__ALIAS_IN_PROJECT_WITH_OVERRIDE_SQL,
            "ref_foo_alias.sql": fixtures.MODELS__REF_FOO_ALIAS_SQL,
        }

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "cast.sql": fixtures.MACROS__CAST_SQL,
            "expect_value.sql": fixtures.MACROS__EXPECT_VALUE_SQL,
        }

    def test_alias_model_name(self, project):
        results = run_dbt(["run"])
        assert len(results) == 4
        run_dbt(["test"])


class BaseAliasErrors:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "config-version": 2,
            "macro-paths": ["macros"],
        }

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "cast.sql": fixtures.MACROS__CAST_SQL,
            "expect_value.sql": fixtures.MACROS__EXPECT_VALUE_SQL,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "model_a.sql": fixtures.MODELS_DUPE__MODEL_A_SQL,
            "model_b.sql": fixtures.MODELS_DUPE__MODEL_B_SQL,
        }

    def test_alias_dupe_thorews_exeption(self, project):
        message = ".*identical database representation.*"
        with pytest.raises(Exception) as exc:
            assert message in exc
            run_dbt(["run"])


class BaseSameAliasDifferentSchemas:
    @pytest.fixture(scope="class")
    def project_config_update(self):
        return {
            "config-version": 2,
            "macro-paths": ["macros"],
        }

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "cast.sql": fixtures.MACROS__CAST_SQL,
            "expect_value.sql": fixtures.MACROS__EXPECT_VALUE_SQL,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "schema.yml": fixtures.MODELS_DUPE_CUSTOM_SCHEMA__SCHEMA_YML,
            "model_a.sql": fixtures.MODELS_DUPE_CUSTOM_SCHEMA__MODEL_A_SQL,
            "model_b.sql": fixtures.MODELS_DUPE_CUSTOM_SCHEMA__MODEL_B_SQL,
            "model_c.sql": fixtures.MODELS_DUPE_CUSTOM_SCHEMA__MODEL_C_SQL,
        }

    def test_same_alias_succeeds_in_different_schemas(self, project):
        results = run_dbt(["run"])
        assert len(results) == 3
        res = run_dbt(["test"])
        assert len(res) > 0


class BaseSameAliasDifferentDatabases:
    @pytest.fixture(scope="class")
    def project_config_update(self, unique_schema):
        return {
            "config-version": 2,
            "macro-paths": ["macros"],
            "models": {
                "test": {
                    "alias": "duped_alias",
                    "model_b": {"schema": unique_schema + "_alt"},
                },
            },
        }

    @pytest.fixture(scope="class")
    def macros(self):
        return {
            "cast.sql": fixtures.MACROS__CAST_SQL,
            "expect_value.sql": fixtures.MACROS__EXPECT_VALUE_SQL,
        }

    @pytest.fixture(scope="class")
    def models(self):
        return {
            "schema.yml": fixtures.MODELS_DUPE_CUSTOM_DATABASE__SCHEMA_YML,
            "model_a.sql": fixtures.MODELS_DUPE_CUSTOM_DATABASE__MODEL_A_SQL,
            "model_b.sql": fixtures.MODELS_DUPE_CUSTOM_DATABASE__MODEL_B_SQL,
        }

    def test_alias_model_name_diff_database(self, project):
        results = run_dbt(["run"])
        assert len(results) == 2
        res = run_dbt(["test"])
        assert len(res) > 0


class TestAliases(BaseAliases):
    pass


class TestAliasErrors(BaseAliasErrors):
    pass


class TestSameAliasDifferentSchemas(BaseSameAliasDifferentSchemas):
    pass


class TestSameAliasDifferentDatabases(BaseSameAliasDifferentDatabases):
    pass
